% Plot power and N_{1/2} for two-class model
%
% Input:
% power_struct - structure with power for each simulation
% num_samples_struct -  structure with number of samples for each simulation
% f_rare -
% f_rare_vec - vector of rare alleles frequencies
% f_rare_power_vec -
% beta_vec - vector of effect sizes
% N_half_mat - matrix of sample size required to achieve 50% power.
% N_half_mat_only_null_enrichment -
% s_null_vec -  vector of selection coefficients
% show_s_null_ind - some plots only show specific values of selection coefficient s
% i_beta - index of effect size
% i_n - index of ...
% title_str - name of ...
% figs_dir - where to save figures
% new_figs_dir - where to save more figures
% n_samples_half_power_vec -
% power_mat -  matrix of power ...
% n_vec -
% new_kept_alleles_vec -
% alpha_vec -  ??
% new_alpha_vec - ??
% rare_cumulative_per_gene - total rate of mutations in one gene
%  demographic_models_struct - new! allow different demographic models (not just equilibrium)
%
% Output: None. We just plot figures
%
function plot_two_class_power_statistics(equilibrium_parameters_output_file, ...
    power_output_dir, new_figs_dir)



% % % % % num_samples_struct, power_struct, ...
% % % % %     two_class_stat_struct, w_x_null_mat, w_x_harmless, w_all, ...
% % % % %     frac_null_by_freq_cumulative, ...
% % % % %     f_rare, f_rare_vec, beta_vec, ...
% % % % %     s_null_vec, show_s_null_ind,  i_beta, i_n, ...
% % % % %     title_str, figs_dir, new_figs_dir, ...
% % % % %     n_vec, new_kept_alleles_vec, alpha_vec, new_alpha_vec, ...
% % % % %     N, prevalence, gene_length, mu, ...
% % % % %     rare_cumulative_per_gene,  demographic_models_struct) % plot power curves


%%%%% OLDD:
%%%%%%%%%%%%%%
% % % % f_rare, f_rare_vec, f_rare_power_vec, beta_vec, ...
% % % %     N_half_mat, N_half_mat_only_null_enrichment, ...
% % % %     s_null_vec, show_s_null_ind,  i_beta, i_n, ...
% % % %     title_str, figs_dir, new_figs_dir, n_samples_half_power_vec, power_mat,  power_struct{3}.power_mat, ...
% % % %     num_samples_struct{2}.num_samples_mat, ... % num_samples_struct{2}.num_samples_mat2, ...
% % % %     n_vec, new_kept_alleles_vec,  alpha_vec, new_alpha_vec, ...
% % % %     rare_cumulative_per_gene)


AssignGeneralConstants;
[mu gene_length mu_per_gene] = set_genomic_mutation_rates(); % set mutation rates
polyphen_false_pos = 0.2; % 0.35; % 0.2;
polyphen_false_neg = 0.2; % 0.35; % 0.2; % set accuracy of functional-allele prediction



desired_power=0.5; % what power do we want? used for sample size estimation
load(equilibrium_parameters_output_file); % load all parameters from input file
if(~ismember('equilibrium', demographic_models_struct.model_str))
    demographic_models_struct.model_str{end+1} = 'equilibrium';
end
demographic_models_struct.num_models = length(demographic_models_struct.model_str);


orange = [1 0.6 0.1]; % set color
population_color_vec = {'k', orange, 'r', 'g', 'b', 'c'};
eric_color_vec = 'kbgyr'; % Conenstion for selection coefficients (we don't have orange. Use yellow)
my_symbol_vec = {'--', '-'}; % flip ordering (set integer powers as solid lines)
selection_color_vec = {'k', 'b', 'g', orange, 'r'}; % replace yellow with orange


f_vec = 0.001; grr_vec = beta_to_genetic_relative_risk(beta_vec, f_vec, prevalence);
[~, i_beta] = min(abs(grr_vec  - 5)); % 61; % choose specific beta. We choose grr to be 5
[~, i_n] = min(abs(n_vec - 50000)); %  25; % choose specific n
[~, f_ind] = min(abs(f_rare - f_rare_vec))
s_ind = show_s_null_ind(1);


% New: add a power text file output
show_s_null = s_null_vec(show_s_null_ind);
cutoff_freq_vec = [0.001 0.01]; % take in order

R_all_models = cell(28,1);

n_null_vec_equilibrium = 2 .* N .* mu .* gene_length .* 2 .* ...
    two_class_stat_struct{1}.normalization_factor_x; % Get equilibrium analytic values


selection_legend_vec = cellstr([repmat('s=10^{', length(show_s_null_ind), 1) ...
    num2str(vec2column(round(2*log10(s_null_vec(show_s_null_ind)))./2), 2) repmat('}', length(show_s_null_ind), 1)]);
selection_legend_vec{1}='s=0'; % replace 10^(-6) by 0

%['s=10^(' num2str(round(2*log10(show_s_null(j)))/2) ')']

alpha_vec=0.25; % 0.1; % 25; % SET DEFAULT FOR PAPER !!! 0.33;  % 0.6; % 0.33; % 0.2; % 0.35; % modify alpha !!
for i_d = 1:demographic_models_struct.num_models
    try
        load(fullfile( power_output_dir, demographic_models_struct.model_str{i_d}, 'two_class_power_parameters.mat')); % load power struct specific for this model
    catch
        continue;
    end
    
    beta_hypomorph = 0.5;  delta_hypomorph = 0.2; % 0.5; % when delta=1 we should get the same curves !!!
    demographic_models_struct = compute_inflation_factor_internal(demographic_models_struct, i_d, ...
        polyphen_false_pos, polyphen_false_neg, ... % set accuracy of functional-allele prediction
        frac_null_by_freq_cumulative, c_cumulative, alpha_vec, beta_hypomorph, delta_hypomorph); % compute multiple additiona parameters
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Compute excel table:
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [R_power new_R_all_models use_f_vec n_null_vec] = ...
        compute_power_table_internal(demographic_models_struct, num_samples_struct, two_class_stat_struct, frac_null_by_freq_cumulative, ...
        demographic_models_struct.data{i_d}.f_null_vec, demographic_models_struct.data{i_d}.rho_vec, i_d, R_all_models,...
        alpha_vec, mu, gene_length, show_s_null, show_s_null_ind, cutoff_freq_vec, new_figs_dir);
    
    
    %    load(power_parameters_output_file);
    R_all_models = new_R_all_models;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % loop on demographic model


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
populations_ordered = {'equil', 'expan1', 'expan2', '2phase', 'europ', 'finn1', 'finn2', 'ice', 'varsel1', 'varsel2', ...
    'finn1_bneck', 'finn1_nobneck', 'finn2_bneck', 'finn2_nobneck', 'ice_bneck', 'ice_nobneck'}; % Get ordering for plotting
[~, populations_perm, temp_perm] = intersect(demographic_models_struct.model_str, populations_ordered);
populations_perm = populations_perm(inv_perm(temp_perm));

if(length(demographic_models_struct.data) < demographic_models_struct.num_models)
    demographic_models_struct.data{demographic_models_struct.num_models} = [];
end
demographic_models_struct.file_names = vec2column(demographic_models_struct.file_names);
demographic_models_struct.model_str = vec2column(demographic_models_struct.model_str);
demographic_models_struct.data = vec2column(demographic_models_struct.data);
demographic_models_struct = order_struct_by_perm(demographic_models_struct, populations_perm, [], 1);  % new ordering
demographic_models_struct.num_models = length(populations_perm);


lambda_vec=demographic_models_struct.data{1}.lambda_vec;
% NEW! plot de-novo power
mu_vec = 10^(-7):10^(-7):10^(-4);
false_positives = 0.05/20000; false_negatives = 0.1;

denovo_lambda_vec = [2 4 9 19 49 99];
plot_two_class_de_novo_power(mu_vec, false_positives, false_negatives, denovo_lambda_vec, new_figs_dir);
show_lambda_vec = [1.5 2 3 5 10 20]-1; % set different relative-risk values. Only strong effects !!!
plot_diruptive_power(mu, s_null_vec, show_lambda_vec, false_positives, false_negatives, new_figs_dir)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot_inflation_factor_equilibrium_analytic_internal(s_null_vec, N, two_class_stat_struct{1}, ...
%     polyphen_false_pos, polyphen_false_neg, ... % set accuracy of functional-allele prediction
%     new_figs_dir);




plot_inflation_factor_internal(demographic_models_struct, show_s_null, show_s_null_ind, ...
    selection_legend_vec, lambda_vec, my_symbol_vec, eric_color_vec, color_vec, ...
    population_color_vec, selection_color_vec, polyphen_false_pos, polyphen_false_neg, new_figs_dir);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plot_sample_size_all_models_together_internal(R_all_models, show_s_null, show_s_null_ind, n_null_vec_equilibrium, new_figs_dir);

close all;

% return;

for i_d = 1:demographic_models_struct.num_models    % New: Plot contour plots for equal sample size as a function of selection coefficient and effect size
    
    % for i_d = 1:demographic_models_struct.num_models
    stat_ctr=1;
    plot_LOF=1; % which alleles to plot?
    if(plot_LOF)
        neutral_freq_per_gene = 0.08;
        allele_str = 'LOF';
    else
        neutral_freq_per_gene = 0.4;
        allele_str = 'NULL'
    end
    
    
    for stat_used = {'LRT_analytic'} %  'eric_crude_enrichment_analytic'} %  'chi-square-analytic'}
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % NEW! Plot just power for different values of GRR. This is HEAVY computation (for each demography)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        plot_power_internal(demographic_models_struct, num_samples_struct, two_class_stat_struct, ...
            i_d, stat_used{1}, stat_ctr, grr_vec, neutral_freq_per_gene, s_null_vec, show_s_null, show_s_null_ind, prevalence, desired_power, ...
            allele_str, color_vec, symbol_vec, new_figs_dir);
        plot_contours_power_internal(demographic_models_struct, num_samples_struct, grr_vec, s_null_vec, show_s_null_ind, desired_power, ...
            stat_used{1}, new_figs_dir);
        stat_ctr=stat_ctr+1;
    end % loop on stat counter
    
    
    continue; % for now don't do other plots below
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %% plot_more_figures_internal();
    
end % loop on demographic models




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal function for computing and saving power in a tab-delimited format
function [R_power R_all_models use_f_vec n_null_vec] = ...
    compute_power_table_internal(demographic_models_struct, num_samples_struct, two_class_stat_struct, frac_null_by_freq_cumulative, ...
    f_null_vec, rho_vec, i_d, input_R_all_models, ...
    alpha_vec, mu, gene_length, show_s_null, show_s_null_ind, cutoff_freq_vec, new_figs_dir)

AssignGeneralConstants;

R_all_models = input_R_all_models;
R_power = cell(20,  length(show_s_null_ind)+1);
R_power{1,1} = ['Model: ' demographic_models_struct.model_str{i_d}];
R_power{1,2} = ['\mu_G=' num2str(mu*gene_length)];
R_power{1,3} = ['\alpha_{birth}=' num2str(alpha_vec)];

ctr=4;     samp_ind = 6;
switch demographic_models_struct.model_str{i_d}
    case 'equilibrium'
        %         rho_vec = frac_null_by_freq_cumulative{1}(show_s_null_ind,:); % fraction of null alleles conditional on allele freq. below f
        %         f_null_vec = c_cumulative{1}(show_s_null_ind,:); % fraction of null alleles which are below freq. f
        use_f_vec = f_rare_vec;
        %         demographic_models_struct.data{i_d}.x_vec = f_rare_vec;
        n_null_vec = 2 .* N .* mu .* gene_length .* 2 .* ...
            two_class_stat_struct{1}.normalization_factor_x; % Get equilibrium analytic values
    otherwise % demographic simulations
        %        demographic_models_struct.data{i_d} = load(demographic_models_struct.file_names{i_d}); % this gives both s-vec and cumulatives
        %         rho_vec = demographic_models_struct.data{i_d}.p_vec(end:-1:1,:) ./ ...
        %             ( demographic_models_struct.data{i_d}.p_vec(end:-1:1,:) + ...
        %             repmat(demographic_models_struct.data{i_d}.p_vec(end,:), 10, 1) ); % This should go DOWN!!!
        %         f_null_vec = bsxfun(@rdivide, demographic_models_struct.data{i_d}.p_vec(end:-1:1,:), ...
        %             demographic_models_struct.data{i_d}.p_vec(end:-1:1,end));  % make sure this is normalized. This should go UP!
        use_f_vec = demographic_models_struct.data{i_d}.x_vec;
        n_null_vec = demographic_models_struct.data{i_d}.p_vec(end:-1:1,end); % take cumulatives
end
cutoff_freq_inds = zeros(length(cutoff_freq_vec), 1);


for j=1:length(show_s_null) % loop on selection coefficients
    R_power{ctr,j+1} = ['s=10^(' num2str(round(2*log10(show_s_null(j)))/2) ')'];  % Get headers
end
ctr=ctr+2;

R_power{ctr,1} = 'Total Allele Freq. n_{null}(1)'; % ctr=ctr+1;
for j=1:length(show_s_null) % loop on selection coefficients
    R_power{ctr,j+1} = num2str(n_null_vec(j), 2);
end
ctr=ctr+2;

R_power{ctr,1} = 'Optimal Threshold T';
for j=1:length(show_s_null) % loop on selection coefficients
    R_power{ctr,j+1} = num2str(demographic_models_struct.data{i_d}.optimal_threshold(1,j), 2); % Take optimal threshold for small lambda
end
ctr=ctr+2;


%demographic_models_struct.data{i_d}.median_DAF=demographic_models_struct.data{i_d}.median_DAF(end:-1:1); % reverse s direction
R_power{ctr,1} = 'Median Allele Frequency f_{1/2}';
for j=1:length(show_s_null) % loop on selection coefficients
    R_power{ctr,j+1} = num2str(demographic_models_struct.data{i_d}.median_DAF(j), 2);
end
ctr=ctr+2;

R_power{ctr,1} = 'Proportion \rho(T)'; ctr=ctr+1;
for i_c = 1:length(cutoff_freq_vec)  % loop on possible frequency cutoffs
    [~, cutoff_freq_inds(i_c)] = min( abs(use_f_vec - cutoff_freq_vec(i_c)) );
    R_power{ctr,1} = ['rho(T=' num2str(cutoff_freq_vec(i_c)*100) '%)'];
    
    for j=1:length(show_s_null) % loop on selection coefficients
        R_power{ctr,j+1} = num2str(rho_vec(j,cutoff_freq_inds(i_c)), 2); % get rho: prob. (null | <= f)
    end
    ctr=ctr+1;
end
R_power{ctr,1} = 'rho(T_{optimal})';
for j=1:length(show_s_null) % Add rho(T_opt)
    R_power{ctr,j+1} = num2str(demographic_models_struct.data{i_d}.optimal_rho(1,j), 2); % Take value for small lambda
    %     [T_min_num_samples(j) T_min_ind(j)] = min( round(0.5*num_samples_struct{samp_ind}.num_samples_mat(1:end-0, j)) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    %     [T_min_num_samples_eric(j) T_min_ind_eric(j)] = min( num_samples_struct{samp_ind+3}.num_samples_mat(1:end-0, j) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    %     R_power{ctr,j+1} = num2str(rho_vec(j, T_min_ind(j)), 3);
end
ctr=ctr+2;



R_power{ctr,1} = 'Cumulative Allele Dist. f_{null}(T)'; ctr=ctr+1;
for i_c = 1:length(cutoff_freq_vec)  % loop on possible frequency cutoffs
    R_power{ctr,1} = ['PSI(T=' num2str(cutoff_freq_vec(i_c)*100) '%)'];
    for j=1:length(show_s_null) % loop on selection coefficients
        R_power{ctr,j+1} = num2str(f_null_vec(j, cutoff_freq_inds(i_c)), 2); % ['s=10^(' num2str(log10(s_null_vec(j))) ')'];
    end
    ctr=ctr+1;
end
R_power{ctr,1} = 'PSI(T_{optimal})';
for j=1:length(show_s_null) % Add T_opt
    R_power{ctr,j+1} = num2str(f_null_vec(j, demographic_models_struct.data{i_d}.optimal_threshold_inds(1,j)), 2); % Take small value of lambda
end
ctr=ctr+2;





R_power{ctr,1} = 'Apparent enrichment for true GRR=5'; ctr=ctr+1;
for i_c = 1:length(cutoff_freq_vec)  % loop on possible frequency cutoffs
    R_power{ctr,1} = ['I(T=' num2str(cutoff_freq_vec(i_c)*100) '%)'];
    for j=1:length(show_s_null) % loop on selection coefficients
        R_power{ctr,j+1} = num2str(1 + (5-1) * rho_vec(j,cutoff_freq_inds(i_c)), 2); % ['s=10^(' num2str(log10(s_null_vec(j))) ')'];
    end
    ctr=ctr+1;
end
R_power{ctr,1} = 'I(T_{optimal})';
for j=1:length(show_s_null) % Add T_opt
    [T_min_num_samples T_min_ind] = min( round(0.5*num_samples_struct{samp_ind}.num_samples_mat(1:end-0, j)) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    [T_min_num_samples_eric T_min_ind_eric] = min( num_samples_struct{samp_ind+3}.num_samples_mat(1:end-0, j) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    R_power{ctr,j+1} = num2str(1 + (5-1) * rho_vec(j,T_min_ind), 2); %
end


ctr=ctr+2;
R_power{ctr,1} = 'Required Sample Size (50% power, pval<2.5*10^6, GRR=5, prev.=5%). Num. cases required assuming either:  Chi-square, equal #controls, or: (Eric''s Zscore, with inifinite #controls)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (missense, perfect knowledge)'; ctr=ctr+1;  % add missense and LOF separately
R_power{ctr,1} = 'SS (missense<= 1%)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (missense<= 0.1%)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (missense<= T_{optimal})'; ctr=ctr+1;
R_power{ctr,1} = 'SS (all LOF, perfect knowledge)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (all LOF + missense<= 1%)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (all LOF + missense<= 0.1%)'; ctr=ctr+1;
R_power{ctr,1} = 'SS (all LOF + missense<= T_{optimal})'; ctr=ctr+1;
R_power{ctr,1} = 'T_{optimal}'; ctr=ctr+1;
R_power{ctr,1} = 'SS (ratio:  missense<= T_{optimal} / perfect knowledge)'; ctr=ctr-9;

for j=1:length(show_s_null) % loop on selection coefficients and compute power
    j_is = j
    i_d_is = i_d
    % num_samples_struct{6} % here we have mixture of nulls and non-nulls
    % num_samples_struct{8} % here we have pure nulls
    
    R_power{ctr,j+1} = [num2str(round(0.5*num_samples_struct{samp_ind+1}.num_samples_mat(end, j))) ...  % sample size for pure missense
        ' (' num2str(num_samples_struct{samp_ind+4}.num_samples_mat(end, j)) ')'];  % show_s_null_ind(j)))
    R_power{ctr+4,j+1} = [num2str(round(0.5*num_samples_struct{samp_ind+2}.num_samples_mat(end, j)), 2) ...  % sample size for pure missense
        ' (' num2str(num_samples_struct{samp_ind+5}.num_samples_mat(end, j)) ')']; ctr=ctr+1;
    for i_c = 1:length(cutoff_freq_vec)  % loop on possible frequency cutoffs
        i_c_is = i_c
        [~, cur_f_ind] = min( abs( use_f_vec - cutoff_freq_vec(i_c) ) ); % find index in
        R_power{ctr,j+1} = [num2str(round(0.5*num_samples_struct{samp_ind}.num_samples_mat(cur_f_ind, j))) ... % sample size for dilluted missense, different cutoffs
            ' (' num2str(num_samples_struct{samp_ind+3}.num_samples_mat(cur_f_ind, j)) ')']; % show_s_null_ind(j)));
        R_power{ctr+4,j+1} = [num2str(round(harmmean([0.5*num_samples_struct{samp_ind}.num_samples_mat(cur_f_ind, j) ...
            0.5*num_samples_struct{samp_ind+2}.num_samples_mat(end, j)])/2)) ...
            ' (' num2str(round(harmmean([ 0.5*num_samples_struct{samp_ind+3}.num_samples_mat(cur_f_ind, j) ...
            num_samples_struct{samp_ind+5}.num_samples_mat(end, j)])/2), 2) ')'];  ctr=ctr+1;
    end
    [T_min_num_samples T_min_ind] = min( round(0.5*num_samples_struct{samp_ind}.num_samples_mat(1:end-0, j)) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    [T_min_num_samples_eric T_min_ind_eric] = min( num_samples_struct{samp_ind+3}.num_samples_mat(1:end-0, j) ); % show_s_null_ind(j))); % get threshold achieving maximal power (minimal sample size)
    R_power{ctr,j+1} = [num2str(T_min_num_samples) ' (' num2str(T_min_num_samples_eric)  ')']; % sample size for dilluted, optimal threshold
    R_power{ctr+4,j+1} = [num2str( round(harmmean([T_min_num_samples 0.5*num_samples_struct{samp_ind+2}.num_samples_mat(end, j)])/2), 2) ...
        ' (' num2str( round(harmmean([T_min_num_samples_eric num_samples_struct{samp_ind+5}.num_samples_mat(end, j)])/2), 2) ')'];
    ctr=ctr+1; % num samples when taking optimal threshold
    R_power{ctr+4,j+1} = [num2str(100*use_f_vec(T_min_ind), 2) '% (' ...
        num2str(100*use_f_vec(T_min_ind_eric), 2) '%)']; % took optimal threshold. Should be: use_f_vec
    R_power{ctr+5,j+1} = num2str(str2nums(R_power{ctr-1,j+1}) ./ str2nums(R_power{ctr-4,j+1}), 2); ctr=ctr-4;
    
    
    %         for combined_ctr = 1:4; % here we combine tests. Sample size (approximate) is harmonic mean
    %             R_power{ctr+4+combined_ctr,j} = harmmean(R_power{ctr+4,j+1}, R_power{ctr+combined_ctr,j+1})/2;
    %
    %         end
end  % loop on s_null

R_power = num2str_cell(R_power, 2);
savecellfile(num2str_cell(R_power), fullfile(new_figs_dir, demographic_models_struct.model_str{i_d}, 'power_table.txt'), ...
    [], 1);

R_power{4,1} = 's';
R_power{6,1} = '$f_{null}$'; R_power{8,1} = '$T^*$'; R_power{10,1} = '$f_{1/2}$';
R_power{12,1} = 'Proportion $\rho_s$'; R_power{13,1} = '$\rho_s(0.1\%)$';
R_power{14,1} = '$\rho_s(1\%)$'; R_power{15,1} = '$\rho_s(T^*)$';
R_power{17,1} = 'IAF $\Psi_s$'; R_power{18,1} = '$\Psi_s(0.1\%)$';
R_power{19,1} = '$\Psi_s(1\%)$'; R_power{20,1} = '$\Psi_s(T^*)$';
R_latex = latex(R_power([4 6 10 8 15 20 14 19 13 18],:), 2, 2); % NEW: save in latex % 13:15 18:20
R_latex = mat2cell(R_latex, ones(size(R_latex,1),1));
R_latex{1} = strrep(R_latex{1},  '|c', '|r'); % align to right
R_latex{1} = strrep(R_latex{1},  '{|r', '{|l'); % align to left
R_latex{2} = strrep(strrep(strrep(strrep(R_latex{2}, 's=', '$'), '(', '{'), ')', '}$'), '10^{-6}', '0');
savecellfile(R_latex, fullfile(new_figs_dir, demographic_models_struct.model_str{i_d}, 'power_table_latex_tab.txt'), ...
    [], 1);



power_str_ind = 5;
for i_plot = 1:28 % Copy all to unique structure for all models. New: copy also optimal threshold and rho
    if(length(R_power{power_str_ind+i_plot,1}) < 50)
        R_all_models{i_plot}{1,1} = R_power{power_str_ind+i_plot,1};
    else
        R_all_models{i_plot}{1,1} = R_power{power_str_ind+i_plot,1}(1:20);
        R_all_models{i_plot}{1,2} = R_power{power_str_ind+i_plot,1}(21:end);
    end
    R_all_models{i_plot}{2,1} = 'Model:';
    R_all_models{i_plot}{i_d+2,1} = demographic_models_struct.model_str{i_d};
    for j=1:length(show_s_null)
        R_all_models{i_plot}{2,j+1} = ['s=10^(' num2str(round(2*log10(show_s_null(j)))/2) ')'];  % Get headers
        R_all_models{i_plot}{i_d+2, j+1} = R_power{power_str_ind+i_plot, j+1};
    end
end



%%%%%%%%%%% End preparing table %%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Plot more figures for each demographic model (currently not used) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_more_figures_internal()
ctr=5;

figure; % Plot power for different values of s
y_lim = [min( min(min(num_samples_struct{ctr+1}.num_samples_mat(:,:), ... % show_s_null_ind))), ...
    min(min(num_samples_struct{ctr+2}.num_samples_mat(:,:))) ), ... % show_s_null_ind))) ) ...
    max( max(max(num_samples_struct{ctr+1}.num_samples_mat(:,:))), ... % show_s_null_ind))), ...
    max(max(num_samples_struct{ctr+2}.num_samples_mat(:,:))) ) ) )]; % show_s_null_ind))) )];
y_lim(1) = 10^(floor(log10(y_lim(1))));
y_lim(2) = 10^(ceil(log10(y_lim(2))));
for i_p=1:2 % plot N_1/2 for different values of s and f^*
    for log_x_flag = 1:1 % 0:1 % 1:1 % plot only on log scale
        subplot(1,2, i_p);
        %            subplot(2,2, i_p + log_x_flag*2);
        switch log_x_flag
            case 0
                log_x_str = '';
                semilogy(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(num_samples_struct{ctr+i_p,q}.num_samples_mat(:,:)), ... % show_s_null_ind)), ...
                    'linewidth', 2); hold on; % 0.5 is because we look at people, not chromosomes
                semilogy(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(num_samples_struct{ctr+2+i_p}.num_samples_mat(:,:)), ... % show_s_null_ind)), ...
                    '--', 'linewidth', 2); % 0.5 is because we look at people, not chromosomes
                %                 semilogy(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(N_half_mat_only_null_enrichment2{i_p}(:,show_s_null_ind)), ...
                %                     ':', 'linewidth', 2); % 0.5 is because we look at people, not chromosomes
                
            case 1
                log_x_str = ' (log)';
                loglog(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(num_samples_struct{ctr+i_p}.num_samples_mat(:,:)), ... % show_s_null_ind)), ...
                    'linewidth', 2); hold on; % 0.5 is because we look at people, not chromosomes
                loglog(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(num_samples_struct{ctr+2+i_p}.num_samples_mat(:,:)), ... % show_s_null_ind)), ...
                    '--', 'linewidth', 2); % 0.5 is because we look at people, not chromosomes
                %                 loglog(f_rare_vec, beta_vec(i_beta)^2 .* 0.5 .*(N_half_mat_only_null_enrichment2{i_p}(:,show_s_null_ind)), ...
                %                     ':', 'linewidth', 2); % 0.5 is because we look at people, not chromosomes
        end
        xlabel(['allele freq. cutoff' log_x_str]); ylabel('Sample size (1/2 power) X \beta^2 (log)');
        xlim([0 0.95]);
        cur_title_str = strrep(title_str, 'Detect. power rare.', 'N_{1/2} \times \beta^2, ')
        cur_title_str = strrep(cur_title_str, 'f^*=1%,', '');
        cur_title_str = strdiff(cur_title_str, 'cum. freq.,');
        %            title(cur_title_str);
    end % loop on log flag
    
    % %         figure; % Make contour plot of N_{1/2}
    % %         [C,h] = contour(s_null_vec, f_rare_vec, N_half_mat);
    % %         set(h,'ShowText','on','TextStep',get(h,'LevelStep'))
    % %         colormap cool;
    % %         xlabel('Selection s'); ylabel('Allele freq. f^*');
    % %         title('Sample size needed to acheieve 1/2 power');
    % %         my_saveas(gcf, fullfile(figs_dir, 'power', 'rare_variants_power_contour_as_function_of_effect_size'), ...
    % %         {'epsc', 'pdf'});
    %    y_lim = get(gca, 'ylim');
    if(y_lim(2) > y_lim(2))
        ylim(y_lim); % temp !!!
    end
end % loop on p-value
cur_title_str = ['N_{1/2} \times \beta^2,  \alpha=33.3% null, ' ...
    'c=100%, p-val cutoff =  0.5 (left), 2.5e-006 (right)'];
%    suptitle(cur_title_str);
legend_vec = [repmat('s^*= -', length(show_s_null_ind), 1) num2str(vec2column(s_null_vec(show_s_null_ind)), 2)];


legend(legend_vec,1); % 3);
my_saveas(gcf, fullfile(new_figs_dir, 'power', ...
    'rare_variants_sample_size_N_0.5_as_function_of_s_and_f_star'), {'epsc', 'pdf'});

%%%%%%%%%Plot Ratio of sample size needed (enrichment vs. no enrichment) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
ratio_vec =  num_samples_struct{ctr+2}.num_samples_mat(:,show_s_null_ind) ./  num_samples_struct{ctr+2+2}.num_samples_mat(:,show_s_null_ind);
[min_sample_size min_sample_size_inds] = min(num_samples_struct{ctr+2}.num_samples_mat(:,show_s_null_ind));
loglog(f_rare_vec, ratio_vec, 'linewidth', 2); hold on; % 0.5 is because we look at people, not chromosomes
plot(f_rare_vec(min_sample_size_inds), diag(ratio_vec(min_sample_size_inds,:)), '*k', 'markersize', 8);
xlabel(['allele freq. cutoff' log_x_str]); ylabel('Fold-Saving by Enrichment');
legend(legend_vec,2); % 3);
my_saveas(gcf, fullfile(new_figs_dir, 'power', ...
    'rare_variants_sample_size_N_0.5_enrichment_effect_as_function_of_s_and_f_star'), {'epsc', 'pdf'});

optimal_ratio_vec = min(num_samples_struct{ctr+2}.num_samples_mat(:,show_s_null_ind)) ./ min(num_samples_struct{ctr+2+2}.num_samples_mat(:,show_s_null_ind));
figure; semilogx(s_null_vec(show_s_null_ind), optimal_ratio_vec, '*');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure; plot(beta_vec, num_samples_struct{1}.num_samples_mat); title('# samples needed as function of \beta');
xlabel('\beta'); ylabel('N_{1/2}');


beta_vec_str = beta_vec-1; % string to display
figure; hold on; imagesc_with_labels(power_struct{1}.power_mat(end:-1:1,:), n_vec, beta_vec(end:-1:1), 0.5, [], 10);     % Plot power
ylabel('Effect size \beta'); xlabel('Sample size');
title(title_str);

figure; % Make contour plot of power. Plot equi-power curves as function of samples and effect size plain
[C,h] = contour((n_vec(1:51)), (beta_vec+1), power_struct{1}.power_mat(:,1:51));
set(h,'ShowText','on','TextStep',get(h,'LevelStep'))
colormap cool;
xlabel('Sample size (n)'); ylabel('Effect size (\beta)');
title(title_str);
my_saveas(gcf, fullfile(figs_dir, 'power', 'rare_variants_power_contour_as_function_of_effect_size'), ...
    {'epsc', 'pdf'});


figure; plot(f_rare_vec, power_struct{2}.power_mat, 'linewidth', 2);
title([ strdiff(title_str,  ['f^*=' num2str(f_rare*100,2) '%, ']) ', n=' num2str(n_vec(i_n))]);
xlabel('allele freq. cutoff f^*'); ylabel('Power');
my_saveas(gcf, fullfile(figs_dir, 'power', 'rare_variants_power_as_function_of_allele_freq_cutoff'), ...
    {'epsc', 'pdf'});







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute power when enriching for functional 'null' alleles.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure; hold on; % Make contour plot of power
[C,h] = contour(new_kept_alleles_vec, new_alpha_vec, power_struct{3}.power_mat);
set(h,'ShowText','on','TextStep',get(h,'LevelStep'))
colormap cool;
plot(new_kept_alleles_vec, alpha_vec ./ new_kept_alleles_vec, '--', 'linewidth', 3); % plot feasability boundary
plot(1, alpha_vec, '*', 'markersize', 20); % plot current scenario
plot(alpha_vec, 1, '*r', 'markersize', 20); % plot best scenario
xlabel('Fraction of alleles retained (r)'); ylabel('Fraction of null alleles out of retained');

title(['Power as function of enrichment \alpha and fraction of retained alleles r. Sample size=' ...
    num2str(n_vec(i_n)) ', c=' num2str(rare_cumulative_per_gene*100,3) '%, \beta=' num2str(beta_vec(i_beta), 3)]);
legend('power', 'best feasible curve', 'all alleles', 'only functional alleles');
my_saveas(gcf, fullfile(figs_dir, 'power', 'rare_variants_power_contour_as_function_of_enrichment_alpha'), ...
    {'epsc', 'pdf'});


figure; hold on; % Make contour plot of N_{1/2}, as function of ...
[C,h] = contour(new_kept_alleles_vec, new_alpha_vec, log10(num_samples_struct{2}.num_samples_mat));
set(h,'ShowText','on','TextStep',get(h,'LevelStep'))
colormap cool;
plot(new_kept_alleles_vec, alpha_vec ./ new_kept_alleles_vec, '--', 'linewidth', 3); % plot feasability boundary
plot(new_kept_alleles_vec, repmat(alpha_vec, length(new_kept_alleles_vec), 1), '--g', 'linewidth', 3); % plot feasability boundary
plot(1, alpha_vec, '*', 'markersize', 20); % plot current scenario
plot(alpha_vec, 1, '*r', 'markersize', 20); % plot best scenario
xlabel('Fraction of alleles retained (r)'); ylabel('Fraction of null alleles out of retained');
%title(['N_{1/2} as function of enrichment \alpha and fraction of retained alleles r. Sample size=' ...
%    num2str(n_vec(i_n)) ', c=' num2str(rare_cumulative_per_gene*100,3) '%, \beta=' num2str(beta_vec(i_beta), 3)]);
legend('N_{1/2}', 'best feasible curve', 'lower-bound curve', 'all alleles', 'only functional alleles');
my_saveas(gcf, fullfile(figs_dir, 'power', 'rare_variants_N_50_contour_as_function_of_enrichment_alpha'), ...
    {'epsc', 'pdf'});













%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Internal Plot Function 4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function plot_contours_power_internal(demographic_models_struct, num_samples_struct, grr_vec, s_null_vec, show_s_null_ind, desired_power, ...
    stat_used, new_figs_dir)

stat_str = str2word('-', strrep(stat_used, '_', '-'), 1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot_contour=0;
if(plot_contour)  % Plot contours - not used for now
    figure; [C,h] = contour(s_null_vec(show_s_null_ind), grr_vec, ...
        log10(num_samples_struct{4+stat_ctr-1}.num_samples_mat(1:10,:)'), 1:0.5:10, ...
        'linewidth', 2); % 5 is for my power statistic, 6 is for Eric's crude statistic
    % [C,h] = contour(new_kept_alleles_vec, new_alpha_vec, power_struct{3}.power_mat);
    set(h,'ShowText','on','TextStep',get(h,'LevelStep'));
    th = clabel(C,h) % change text to include powers of 10
    for i=1:0.5:10
        set(findobj(th,'String',num2str(i)),'String',['10^{' num2str(i) '}'])
    end
    set(gca,'xscale','log'); % set(gca, 'yscale', 'log');
    colormap cool;
    xlabel('s'); ylabel('GRR');
    title(['Isocurves for $N_{' num2str(desired_power*100) '\%}$. $p_{\alpha}=' num2str(num_samples_struct{4}.p_val_cutoff) '$. $\pi=' num2str(prevalence) '$. ' ...
        demographic_models_struct.model_str{i_d} '. stat=' stat_str '. ' allele_str ' alleles'], 'interpreter', 'latex');
    ylim([1 10]);
    add_faint_grid(0.5);
    
    % % % figure; my_contour(log10(s_null_vec), grr_vec, log10(num_samples_struct{4}.num_samples_mat'), 1:0.5:5);
    % % % ylim([1 10]); legend( num2str(  (1:0.5:5)' ) )
    
    %         my_saveas(gcf, fullfile(new_figs_dir, demographic_models_struct.model_str{i_d}, ...
    %             ['fig1c_sample_size_gene_burden_test_function_of_GRR_and_s_isocurves_' stat_used]), ...
    %             {'epsc', 'pdf'});
    my_saveas(gcf, fullfile(new_figs_dir, 'power/isocurves', ...
        [demographic_models_struct.model_str{i_d} '_sample_size_isocurves_' stat_used]), ...
        {'epsc', 'pdf'});
end % if plot contour




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Internal Plot Function 5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Plot inflation in sample size resulting from adding missense alleles to LOF alleles
function plot_inflation_factor_internal(demographic_models_struct, show_s_null, show_s_null_ind, ...
    selection_legend_vec, lambda_vec, my_symbol_vec, eric_color_vec, color_vec, ...
    population_color_vec, selection_color_vec, polyphen_false_pos, polyphen_false_neg, new_figs_dir)

my_contour_symbol_vec = {'-', '--'};
%my_models_color_vec = 'brgmcbrgmcbrgmcbrgmcb'; % remove black 'k' (reserved for contours)

%lambda=lambda_vec;
%NEW: Plot also inflation factor in LODs (NOT sample size!!!)


% New: plot inflation factor as function of threshold f for different values of s and different demographic models
for i_lambda=8:8 % 1:length(lambda_vec)
    model_legend = [];
    for j=1:0 % length(show_s_null) % First plot one figure per s
        for include_LOF_flag = 0 % :1 % 0: plot I_T or, 1: I_star_T (latter includes also LOFs)
            figure; model_ctr=1; model_legend = [];
            for i_d = 1:demographic_models_struct.num_models
                if(ismember(demographic_models_struct.model_str{i_d}, {'equil', 'expan1', 'expan2', 'europ', '2phase', 'ice', 'finn1', 'finn2'}))
                    if(include_LOF_flag)
                        cur_inflation_factor_vec = ...
                            harmmean([(5/4).*demographic_models_struct.data{i_d}.sample_size_inflation_factor(j,:);  ...
                            repmat(5, 1, length(demographic_models_struct.data{i_d}.x_vec))])./2;
                        inflation_str = '_star'; inflation_str_numeric = '^*';
                    else
                        cur_inflation_factor_vec = (4/4).*demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(j,:); % why 5/4 ?
                        inflation_str = ''; inflation_str_numeric = '';
                    end
                    
                    semilogx(demographic_models_struct.data{i_d}.x_vec, cur_inflation_factor_vec, ...
                        color_vec(model_ctr), 'linewidth', 2);  hold on;
                    model_legend = [model_legend {get_nice_population_names(demographic_models_struct.model_str{i_d})}];
                    model_ctr = model_ctr+1;
                end
            end
            %         if(~include_LOF_flag)
            %             ylim([1 10]);
            %         else
            %             ylim([1 5.01]);
            %         end
            xlabel('Allele frequency threshold T', 'fontsize', 15, 'fontweight', 'bold');
            ylabel('Relative value of missense alleles I(T)', 'fontsize', 15, 'fontweight', 'bold'); % Sample Size Inflation Factor');
            title(['Inflation $I' inflation_str_numeric '(T)$ in Sample size as function of thereshold. $s=10^{' ...
                num2str(round(2*log10(show_s_null(j)))/2) '}$'], 'interpreter', 'latex');
            %     legend_vec = [repmat('s= 10^{', length(show_s_null), 1) num2str(log10(show_s_null'),3) ...
            %         repmat('}', length(show_s_null), 1)];
            %     legend_vec = cellstr(legend_vec); legend_vec = strrep_cell(legend_vec, ' ', ''); legend_vec{1} = 's= 0'; % fix s=0
            legend(model_legend); legend ('boxoff');
            my_saveas(gcf, fullfile(new_figs_dir, 'all_models', ['I' inflation_str '_T'], ...
                strrep(['power_inflation_factor_I' inflation_str '_T_s_10_'  num2str(round(2*log10(show_s_null(j)))/2)], '.', '_' )), ...
                {'epsc', 'pdf'});
        end % loop on include LOF flag
    end % loop on selection coefficient
    
    supp_info=1; % Plot supp. info.
    model_ctr=1; % count number of models we plot
    for i_d = 1:demographic_models_struct.num_models  % Now plot one figure per demographic model
        if(~ismember(demographic_models_struct.model_str{i_d}, {'equil', 'expan1', 'europ', '2phase', 'ice', 'finn1'})) % 'expan2', , 'finn2'
            continue;
        end
        for include_LOF_flag = 0 % :1 % plot I_T or I_star_T (latter includes also LOFs)
            if((~supp_info) || (model_ctr == 1))
                figure;
            end
            if(supp_info)
                switch model_ctr
                    case 1
                        plot_model_ctr=1;
                    case 2
                        plot_model_ctr=3;
                    case 3
                        plot_model_ctr=5;
                    case 4
                        plot_model_ctr=2;
                    case 5
                        plot_model_ctr=4;
                    case 6
                        plot_model_ctr=6;
                end
                subplot(3,2, plot_model_ctr);
                
            end
            for j=1:length(show_s_null)
                if(include_LOF_flag)
                    cur_inflation_factor_vec = ...
                        harmmean([(5/4).*demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(j,:);  ...
                        repmat(5, 1, length(demographic_models_struct.data{i_d}.x_vec))])./2;
                    inflation_str = '_star'; inflation_str_numeric = '^*';
                else
                    cur_inflation_factor_vec = demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(j,:); % no need for (5/4).*
                    inflation_str = ''; inflation_str_numeric = '';
                end
                
                semilogx(demographic_models_struct.data{i_d}.x_vec, cur_inflation_factor_vec, ...
                    'color', selection_color_vec{ceil(j/2)}, 'linestyle', my_symbol_vec{mod_max(j,2)}, ...
                    'linewidth', 2);  hold on;
                % % %              [eric_color_vec(ceil(j/2)) my_symbol_vec{mod_max(j,2)}], 'linewidth', 2);  hold on;
                %            model_legend = [model_legend {demographic_models_struct.model_str{i_d}}];
            end
            
            if(~include_LOF_flag)
                %            ylim([1 10]);
                ylim([0 1.5]); % 1.5 good for alpha=0.25. Use 2 for higher alpha why so high? 3.2
            else
                ylim([0.5 5.01]);
            end
            xlim([10^(-5) 1]);
            
            if(~supp_info)
                xlabel('Allele frequency threshold T', 'fontsize', 15, 'fontweight', 'bold');
                ylabel('Relative value of missense alleles I(T)', 'fontsize', 15, 'fontweight', 'bold'); % Sample Size Inflation Factor');
                title(get_nice_population_names(demographic_models_struct.model_str{i_d}), 'fontsize', 14);
            else % here one figure for supp info
                if(ismember(plot_model_ctr, [5, 6]))
                    xlabel('Allele frequency threshold T'); % , 'fontsize', 15, 'fontweight', 'bold');
                end
                if(ismember(plot_model_ctr, [3, 4]))
                    ylabel('Relative value of missense alleles I(T)'); % , 'fontsize', 15, 'fontweight', 'bold'); % Sample Size Inflation Factor');
                end
                title(get_nice_population_names(demographic_models_struct.model_str{i_d})); % , 'fontsize', 14);
            end % if ~supp_info
            
            %             title(['ELODs Missense compared to LOF $I' inflation_str_numeric '(T)$ as function of thereshold. ' ...
            %                 demographic_models_struct.model_str{i_d} ', $1+\lambda=' num2str(1+lambda_vec(i_lambda)) '$'], ...,
            %                 'interpreter', 'latex', 'fontsize', 14);  % No title !!!
            
            %            legend(strrep_cell(strrep_cell(selection_legend_vec, 's=', ''), ' ', ''), 2, 'fontsize', 10); legend('boxoff');
            
            if(~supp_info)
                set(gca, 'fontsize', 15); % Problem?
            end
            add_faint_grid(0.5);
            if(~supp_info)
                my_saveas(gcf, fullfile(new_figs_dir, demographic_models_struct.model_str{i_d}, ['I' inflation_str '_T'], ...
                    strrep(['power_inflation_factor_I' inflation_str '_T_lambda_' num2str(lambda_vec(i_lambda))], '.', '_' )), ...
                    {'epsc', 'pdf', 'jpg'});
            end
        end % loop on include LOF flag
        model_ctr = model_ctr+1;
        
    end % loop on demographic model
    orient landscape;
    
    if(supp_info)
        %                   legend(strrep_cell(strrep_cell(selection_legend_vec, 's=', ''), ' ', ''), 2, 'fontsize', 10);
        h_l = legend(strrep_cell(selection_legend_vec, ' ', ''), 3, 'fontsize', 10); % 'location', 'east');
        pos_l = get(h_l, 'position'); set(h_l, 'position', [pos_l(1)+0.275 pos_l(2)-0.15 pos_l(3) pos_l(4)]);
        legend('boxoff'); % +plot_type*(~figs_for_paper_flag));
        
        %        add_faint_grid(0.5);
        my_saveas(gcf, fullfile(new_figs_dir, 'all_models', ['I' inflation_str '_T'], ...
            strrep(['all_models_power_inflation_factor_I' inflation_str '_T_lambda_' num2str(lambda_vec(i_lambda))], '.', '_' )), ...
            {'epsc', 'pdf', 'jpg'});
    end
    %    close all;
end % loop on effect size lambda



% NEW! Plot same just for Europe, and with hypomorphs
i_d=strmatch('europ', demographic_models_struct.model_str); % Plot only Europe. Check: is it 5 or 3?
i_lambda=11; % Take 1+lambda = 4 (lambda=3)

figure;
max_inflation_factor = zeros(3, length(show_s_null));
for plot_flag=0:2
    subplot(3,1, plot_flag+1);
    
    for j=1:length(show_s_null)
        switch plot_flag
            case 0 % just missense
                plot_str = 'Missense Null';
                cur_inflation_factor =  ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(j,:);
            case 1 % missense+hypomorph
                plot_str = 'Hypomorph';
                cur_inflation_factor =  ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor_hypomorph{i_lambda}(j,:);
            case 2
                plot_str = 'Missense Null and Hypomorph';
                cur_inflation_factor =  ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor_missense_null_plus_hypomorph{i_lambda}(j,:);
            case 3
                plot_str = 'ratio';
                cur_inflation_factor =  ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor_hypomorph{i_lambda}(j,:) ./ ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(j,:);
        end % switch
        semilogx(demographic_models_struct.data{i_d}.x_vec, cur_inflation_factor, ...
            'color', selection_color_vec{ceil(j/2)}, 'linestyle', my_symbol_vec{mod_max(j,2)}, ...
            'linewidth', 2);  hold on;
        
        max_inflation_factor(plot_flag+1, j) = max(cur_inflation_factor);
    end % loop on s-null
    %        add_faint_grid(0.5);
    title(plot_str); % ['LRT FOR ' plot_str]);
    %pos_l = get(h_l, 'position'); set(h_l, 'position', [pos_l(1)+0.275 pos_l(2)-0.15 pos_l(3) pos_l(4)]);
    %legend('boxoff'); % +plot_type*(~figs_for_paper_flag));
    
    xlim([10^(-6) 1]);
    if(plot_flag==1)
        ylabel('Relative Value of missense alleles');
    end
    if(plot_flag==2)
        xlabel('Allele Frequency Threshold T');
    end
    ylim([0 1.0]);
    add_faint_grid(0.5);
end % loop on plot type
h_leg = legend(strrep_cell(selection_legend_vec, ' ', ''), 4, 'fontsize', 10); % 'location', 'east');
pos_l = get(h_leg, 'position'); set(h_leg, 'position', [pos_l(1)+0.1535 pos_l(2)-0.135 pos_l(3) pos_l(4)]);
set(h_leg,'Xcolor',[0.8 0.8 0.8],'Ycolor',[0.8 0.8 0.8]);


orient landscape;

my_saveas(gcf, fullfile(new_figs_dir, 'all_models', ['I' inflation_str '_T'], ...
    strrep(['all_models_power_inflation_factor_I'  '_T_lambda_' num2str(lambda_vec(i_lambda)) '_hypomorph'], '.', '_' )), ...
    {'epsc', 'pdf', 'jpg'});


% New: Plot optimal power as function of s for all models
alpha_vec = 0.25; % SET DEFAULT FOR PAPER 33; % 0.6; % 0.33; % 0.2; % 0.35; % modify alpha
for i_lambda=8:8 % 1:length(lambda_vec)
    lambda=lambda_vec(i_lambda);
    figure;
    model_ctr=1;
    for i_d = 1:demographic_models_struct.num_models  % Now plot one figure per demographic model
        if(~ismember(demographic_models_struct.model_str{i_d}, {'equil', 'expan1', 'expan2', 'europ', '2phase', 'ice', 'finn1', 'finn2'}))
            continue;
        end
        
        semilogx(show_s_null, demographic_models_struct.data{i_d}.optimal_inflation_factor(i_lambda,:), ...
            'color', population_color_vec{mod_max(model_ctr,4)}, ...
            'linestyle', my_symbol_vec{ceil(model_ctr/4)}, 'linewidth', 2); hold on;
        model_ctr=model_ctr+1;
    end
    xlabel('Selection coefficient s'); ylabel('Optimal E[LODs] ratio');
    legend(model_legend, 3); legend('boxoff');
    title(['Optimal ELODs-missense / ELODs-LOF ratio. \lambda=' num2str(lambda)]);
    my_saveas(gcf, fullfile(new_figs_dir, 'power/all_models', ...
        strrep(['ELODs_optimal_ratio_alpha_' num2str(alpha_vec) '_lambda_' num2str(lambda)], '.', '_')), ...
        {'epsc', 'pdf'});
end % loop on lambda


% New: Plot top selection as function of lambda
figure; model_ctr=1;
for i_d = 1:demographic_models_struct.num_models  % Now plot one figure per demographic model
    if(~ismember(demographic_models_struct.model_str{i_d}, {'equil', 'expan1', 'europ', '2phase', 'ice', 'finn1'}))
        continue;
    end
    plot(lambda_vec, demographic_models_struct.data{i_d}.optimal_inflation_factor(:,end), ...
        'color', population_color_vec{model_ctr}, ...
        'linestyle', my_symbol_vec{ceil(model_ctr/4)}, 'linewidth', 2); hold on;  % take last index: strongest selection coefficient
    model_ctr=model_ctr+1;
end % loop on demographic models
xlabel('Effect size 1+\lambda'); ylabel('Optimal value of missense alleles I(T^*)'); % E[LODs] ratio for strong s
legend(model_legend, 4); legend('boxoff');
% title(['Optimal ELODs ratio as function of effect size \lambda. (s=0.1, \alpha=' num2str(alpha_vec) ')']); % No title!!
my_saveas(gcf, fullfile(new_figs_dir, 'power/all_models', ...
    strrep(['ELODs_optimal_ratio_alpha_' num2str(alpha_vec) '_vs_lambda'], '.', '_')), {'epsc', 'pdf', 'jpg'});


% Plot the same curve for all models, but with polyphen !!
for pop_groups = { {'expan1', '2phase', 'europ'} } % Just Euorope % 'equil',   'expan2', '2phase'}, {'europ', 'ice', 'finn1', 'finn2'} }
    if(ismember('europ', pop_groups{1}))
        pop_str = 'standard'; % 'expansion';
    else
        if(ismember('ice', pop_groups{1}))
            pop_str = 'bottleneck';
        else
            if(ismember('finn1_bneck', pop_groups{1}))
                pop_str = 'Finland';
            else
                if(ismember('ice_bneck', pop_groups{1}))
                    pop_str = 'Iceland';
                else
                    pop_str = 'varsel';
                end
            end
        end
    end
    i_s=8; % Choose which s to display: i_s=8 matches s=0.01
    figure; model_ctr=1;         cur_model_legend = [];
    for i_d = 1:demographic_models_struct.num_models  % Now plot one figure per demographic model
        if(~ismember(demographic_models_struct.model_str{i_d}, pop_groups{1})) %  {'equil', 'expan1', 'expan2', 'europ', '2phase', 'ice', 'finn1', 'finn2'}))
            model_ctr=model_ctr+1; % still increase counter
            continue;
        end
        switch demographic_models_struct.model_str{i_d}
            case 'expan1'
                model_ctr = 2; % red
            case '2phase'
                model_ctr = 3; % green
            case 'europ'
                model_ctr = 4; % black
        end % switch model
        plot(1+lambda_vec, demographic_models_struct.data{i_d}.optimal_inflation_factor(:,i_s), ...
            'color', population_color_vec{model_ctr}, 'linewidth', 2); hold on;  % take last index: strongest selection coefficient
        plot(1+lambda_vec, demographic_models_struct.data{i_d}.optimal_inflation_factor_polyphen(:,i_s), ...
            'color', population_color_vec{model_ctr}, 'linestyle', '--', 'linewidth', 2); hold on;  % take last index: strongest selection coefficient
        model_ctr=model_ctr+1;
        cur_model_legend = [cur_model_legend ...
            {get_nice_population_names(demographic_models_struct.model_str{i_d}), ''}];
        %         cur_model_legend = [cur_model_legend ...
        %             {[strrep(demographic_models_struct.model_str{i_d}, 'n1', 'n') ', all missense'], ...
        %             [strrep(demographic_models_struct.model_str{i_d}, 'n1', 'n') ', predicted-nulls(' num2str(polyphen_false_pos*100) '%,' num2str(polyphen_false_neg*100) '%)']}];
    end % loop on demographic models
    xlabel('Effect size 1+\lambda', 'fontsize', 15, 'fontweight', 'bold'); ylabel('Optimal value of missense alleles I(T^*)', 'fontsize', 15, 'fontweight', 'bold');
    xlim([1 20.1]);
    set(gca, 'fontsize', 15);
    x_tick = get(gca, 'XTick');
    if(x_tick(1) > 1)
        x_tick = [1 x_tick];
        set(gca, 'XTick', x_tick);
    end
    
    add_faint_grid(0.5);
    legend(cur_model_legend, 4); legend('boxoff');
    %    title(['Optimal ELODs ratio vs. effect size \lambda. (s=' num2str(show_s_null(i_s), 2) ', \alpha=' num2str(alpha_vec) ') with functional prediction'], 'fontsize', 15);
    % No title !!
    my_saveas(gcf, fullfile(new_figs_dir, 'power/all_models', ...
        strrep(['ELODs_optimal_ratio_alpha_' num2str(alpha_vec) '_vs_lambda_polyphen_' pop_str], '.', '_')), {'epsc', 'pdf'});
end % loop on population groups




% New: make contour plots for all models
all_rho_vec = logspace(-6,0, 500); all_psi_vec = logspace(-6,0,600);
num_rho = length(all_rho_vec); num_psi = length(all_psi_vec);
lambda_vec = [0.1 0.3 0.5 0.75 1 1.5 2 3 5 8 10 15 20]; % get a dense array of lambda values
for i=1:length(lambda_vec)
    ELODs_ratio_mat{i} = approximate_ELODs_ratio_internal(repmat(all_psi_vec, num_rho, 1), ...
        repmat(all_rho_vec', 1, num_psi), lambda_vec(i)); % compute one time in matrix form
end

% New: plot effect of lambda for different demographic models (assuming strong selection, i.e. small f)
for i_lambda = 8:8 % 1:1 % length(lambda_vec) % plot just for small lambda
    lambda = lambda_vec(i_lambda);
    for i_s=8:8 % 1:10 % loop on s
        for plot_polyphen = 0:1
            figure;
            model_legend = [];
            model_ctr=1;
            for i_d = 5 % 1:demographic_models_struct.num_models  % Now plot one figure per demographic model % 5 just europe
                if(~ismember(demographic_models_struct.model_str{i_d}, {'equil', 'expan1', 'europ', '2phase', 'ice', 'finn1'})) % 'expan2', % , 'finn2'
                    continue;
                end
                model_ctr = strmatch(demographic_models_struct.model_str{i_d}, ...
                    {'equil',  'expan1', '2phase', 'europ', 'finn1', 'ice'}); % get correct model index
                model_legend = [model_legend {get_nice_population_names(demographic_models_struct.model_str{i_d})}];
                % Compute for the optimal thresholds the values at strongest s: s=0.1;
                plot([0 demographic_models_struct.data{i_d}.f_null_vec(i_s,:)], ...
                    [alpha_vec ./ (1-alpha_vec) demographic_models_struct.data{i_d}.rho_vec(i_s,:) ./ ...
                    (1-demographic_models_struct.data{i_d}.rho_vec(i_s,:))], ...
                    'color', 'b', ... % population_color_vec{model_ctr}, ...
                    'linestyle', '--', 'linewidth', 2);     hold on; % my_contour_symbol_vec{ceil(model_ctr/4)} % here plot without polyphen
                
                if(plot_polyphen)
                    plot([0 (1-polyphen_false_neg) .* demographic_models_struct.data{i_d}.f_null_vec(i_s,:)], ...
                        [alpha_vec .* (1-polyphen_false_neg) ./ ((1-alpha_vec).*polyphen_false_pos ) ...
                        ((1-polyphen_false_neg)/polyphen_false_pos) .* ...
                        demographic_models_struct.data{i_d}.rho_vec(i_s,:) ./ ...
                        (1-demographic_models_struct.data{i_d}.rho_vec(i_s,:))], ...
                        'color', 'r', ... % population_color_vec{model_ctr}, ...
                        'linestyle', '--', 'linewidth', 2);     hold on; % my_contour_symbol_vec{ceil(model_ctr/4)} % here plot without polyphen
                end
                %            model_ctr=model_ctr+1;
            end
            
            model_ctr=1; % Plot top point
            for i_d = 5 % 1:demographic_models_struct.num_models  % Now plot one figure per demographic model
                if(~ismember(demographic_models_struct.model_str{i_d}, {'equil',  'expan1', '2phase', 'europ', 'finn1', 'ice'}))
                    continue;
                end
                model_ctr = strmatch(demographic_models_struct.model_str{i_d}, ...
                    {'equil',  'expan1', '2phase', 'europ', 'finn1', 'ice'}); % get correct model index
                
                % Compute for the optimal thresholds the values at strongest s: s=0.1;
                plot(demographic_models_struct.data{i_d}.optimal_f_vec(i_lambda, i_s), ...
                    demographic_models_struct.data{i_d}.optimal_rho(i_lambda, i_s) ./ ...
                    (1-demographic_models_struct.data{i_d}.optimal_rho(i_lambda, i_s)), ...
                    'color', 'b', ... % population_color_vec{model_ctr}, ...
                    'linestyle', '*', 'markersize', 10);
                plot(demographic_models_struct.data{i_d}.optimal_f_vec(i_lambda, i_s), ...
                    demographic_models_struct.data{i_d}.optimal_rho(i_lambda, i_s) ./ ...
                    (1-demographic_models_struct.data{i_d}.optimal_rho(i_lambda, i_s)), ...
                    'color', 'b', ... % population_color_vec{model_ctr}, ...
                    'linestyle', 'o', 'markersize', 10);
                
                
                if(plot_polyphen)
                    plot(demographic_models_struct.data{i_d}.optimal_f_vec_polyphen(i_lambda, i_s), ...
                        demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda, i_s) ./ ...
                        (1-demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda, i_s)), ...
                        'color', 'r', ... % population_color_vec{model_ctr}, ...
                        'linestyle', '*', 'markersize', 10);
                    plot(demographic_models_struct.data{i_d}.optimal_f_vec_polyphen(i_lambda, i_s), ...
                        demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda, i_s) ./ ...
                        (1-demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda, i_s)), ...
                        'color', 'r', ... % population_color_vec{model_ctr}, ...
                        'linestyle', 'o', 'markersize', 10);
                end
                
                model_ctr=model_ctr+1;
            end
            good_rho_inds = find(all_rho_vec ./ (1-all_rho_vec) <= 2); % 3
            colormap('gray'); [c, h] = contour(all_psi_vec, all_rho_vec(good_rho_inds) ./ (1-all_rho_vec(good_rho_inds)), ...
                ELODs_ratio_mat{i_lambda}(good_rho_inds,:), 'linewidth', 2); clabel(c, h); colorbar; % choose lambda for plot
            
            xlabel('Frac. captured. \Psi_s(f)'); ylabel('Null vs. Neutral proportion: \rho_s(f) / (1-\rho_s(f))');
            %            title(['E[LODs-missense] / E[LODs-LOF]. $1+\lambda=' num2str(1+lambda) ', ' strrep(selection_legend_vec{i_s}, ' ', '') ...
            %                ', \alpha=' num2str(alpha_vec) '$'], 'interpreter', 'latex', 'fontsize', 16, 'fontweight', 'bold');
            %            legend(model_legend, 3); legend('boxoff');
            
            %        ylim([0 1]);
            if(plot_polyphen)
                my_saveas(gcf, fullfile(new_figs_dir, 'power/all_models', ...
                    strrep(['ELODs_ratio_contours_' strrep(selection_legend_vec{i_s}, ' ', '') '_alpha_' num2str(alpha_vec) '_lambda_' num2str(lambda) '_polyphen'], '.', '_')), ...
                    {'epsc', 'pdf'});
            else
                my_saveas(gcf, fullfile(new_figs_dir, 'power/all_models', ...
                    strrep(['ELODs_ratio_contours_' strrep(selection_legend_vec{i_s}, ' ', '') '_alpha_' num2str(alpha_vec) '_lambda_' num2str(lambda)], '.', '_')), ...
                    {'epsc', 'pdf'});
            end % if polyphen
        end % loop on plot polyphen
    end % loop on s
    %    close all; % too many figures
end % loop on lambda

XXX = 1324124;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Internal Plot Function 6 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




function plot_power_internal(demographic_models_struct, num_samples_struct, two_class_stat_struct, ...
    i_d, stat_used, stat_ctr, grr_vec, neutral_freq_per_gene, s_null_vec, show_s_null, show_s_null_ind, prevalence, desired_power, ...
    allele_str, color_vec, symbol_vec, new_figs_dir)

show_grr_vec = [1.5 2 3 5 10 20]; % set different relative-risk values. Only strong effects !!!

tmp_n_samples_vec = zeros(length(show_grr_vec), length(s_null_vec));
var_explained_mat = zeros(length(show_grr_vec), length(s_null_vec));   % New: compute variance explained
non_centrality_parameter_mat = zeros(length(show_grr_vec), length(s_null_vec));
for plot_type=0:1 % 0 - plot just sample size, 1 - multiply by var-explained
    % NEW! Plot also for protective alleles !!!
    for protective_alleles = 0:1
        if(protective_alleles)
            protective_str = '_protective';
            show_grr_vec = 1 ./ harmful_grr_vec; % take inverse GRR 
        else
            show_grr_vec = harmful_grr_vec; 
            protective_str = '';
        end
        figure;
        if( strcmp(demographic_models_struct.model_str{i_d}, 'equil') ) %%%  && strcmp(stat_used{1}, 'eric_crude_enrichment_analytic') ) % plot also power by theory (just for equilibrium)
            for j=1:length(show_grr_vec)
                run_j = j
                tmp_eric_sample_size = ( norminv(num_samples_struct{4}.p_val_cutoff) - sqrt(show_grr_vec(j)) .* norminv(desired_power) ).^2 ./ ... % why factor 2???? for chromosomes?
                    ((show_grr_vec(j)-1).^2 .* neutral_freq_per_gene .* vec2row(two_class_stat_struct{1}.normalization_factor_x));  % Compute sample size Eric's approximation
                for i_s = 1:length(s_null_vec)
                    sprintf('run s %ld out of %ld', i_s, length(s_null_vec))
                    p_mat = zeros(1,4);
                    p_mat(4) = show_grr_vec(j) * prevalence * neutral_freq_per_gene .* two_class_stat_struct{1}.normalization_factor_x(i_s); % why 0.4?
                    p_mat(2) = prevalence - p_mat(4);
                    p_mat(3) =  neutral_freq_per_gene .* two_class_stat_struct{1}.normalization_factor_x(i_s) - p_mat(4);
                    p_mat(1) = 1-sum(p_mat);
                    if(plot_type==0) % compute everything only first time (second time just plot)
                        var_explained_mat(j,i_s) = p_z_x_marginal_to_heritability(p_mat, 'binary', 'binary');
                        if(min(p_mat) < 0)
                            tmp_eric_sample_size(i_s)  = -1;
                            tmp_n_samples_vec(j, i_s) = -1;
                            continue;
                        end
                        switch stat_used
                            case 'LRT_analytic'
                                [tmp_n_samples_vec(j, i_s) non_centrality_parameter_mat(j, i_s)] = compute_sample_size_from_power( ...
                                    p_mat, desired_power, 1, num_samples_struct{4}.p_val_cutoff, [], 'case-only', 'LRT_analytic', [], [], [], 'bisection'); % NEW! compute sample size
                                %                         [tmp_n_samples_vec2(j, i_s) non_centrality_parameter_mat2(j, i_s)] = compute_sample_size_from_power( ...
                                %                             p_mat, 0.9, 1, num_samples_struct{4}.p_val_cutoff, [], 'case-only', 'LRT_analytic', [], [], [], 'analytic'); % NEW! compute sample size analytically (NOT WORKING!!!)
                        end
                    end
                end
                
                switch stat_used
                    case 'eric_crude_enrichment_analytic'
                        loglog(s_null_vec, tmp_eric_sample_size, [color_vec(j) symbol_vec{stat_ctr}], 'linewidth', 2);  % [color_vec(j) '--']
                    case 'LRT_analytic'
                        if(plot_type == 0)
                            y_vec = tmp_n_samples_vec(j,:);
                            loglog(s_null_vec, y_vec, [color_vec(j) symbol_vec{stat_ctr}], 'linewidth', 2);
                        else
                            y_vec = tmp_n_samples_vec(j,:) .* var_explained_mat(j,:);
                            semilogx(s_null_vec, y_vec, [color_vec(j) symbol_vec{stat_ctr}], 'linewidth', 2);
                        end
                    case 'chi-square-analytic'
                        [~, grr_ind] = min(abs(grr_vec - show_grr_vec(j)));
                        if(plot_type == 0)
                            y_vec =  num_samples_struct{4+stat_ctr-1}.num_samples_mat(1:10, grr_ind);
                            loglog(show_s_null, y_vec, [color_vec(j)  symbol_vec{stat_ctr}], 'linewidth', 2);
                        else
                            y_vec = num_samples_struct{4+stat_ctr-1}.num_samples_mat(1:10, grr_ind) .* vec2column(var_explained_mat(j,show_s_null_ind)); % take sample size times var. explained
                            semilogx(show_s_null, y_vec, [color_vec(j)  symbol_vec{stat_ctr}], 'linewidth', 2);
                        end
                        
                end
                hold on;
            end % loop on j (selection)
        else % here a traditional curve: use empirical data
            if(stat_ctr<=2) % don't take 3rd one!!! (not pre-computed)
                for j=1:length(show_grr_vec)
                    [~, grr_ind] = min(abs(grr_vec - show_grr_vec(j)));
                    
                    if(plot_type==0)
                        for i_s = show_s_null_ind
                            p_mat = zeros(1,4);
                            p_mat(4) = show_grr_vec(j) * prevalence *  neutral_freq_per_gene .* two_class_stat_struct{1}.normalization_factor_x(i_s); % why 0.4?
                            p_mat(2) = prevalence - p_mat(4);
                            p_mat(3) =  neutral_freq_per_gene .* two_class_stat_struct{1}.normalization_factor_x(i_s) - p_mat(4);
                            p_mat(1) = 1-sum(p_mat);
                            var_explained_mat(j,i_s) = p_z_x_marginal_to_heritability(p_mat, 'binary');
                        end
                    end
                    
                    if(plot_type == 0) % Here power is already computed !!! 
                        y_vec = num_samples_struct{4+stat_ctr-1}.num_samples_mat(1:10, grr_ind)
                        loglog(show_s_null, y_vec, [color_vec(j)  symbol_vec{stat_ctr}], 'linewidth', 2); hold on;
                    else
                        y_vec = num_samples_struct{4+stat_ctr-1}.num_samples_mat(1:10, grr_ind) .* vec2column(var_explained_mat(j,show_s_null_ind));
                        semilogx(show_s_null, y_vec, [color_vec(j)  symbol_vec{stat_ctr}], 'linewidth', 2); hold on;
                    end
                end
            end %
        end % if strcmp
        
        
        figure;
        h1 = loglog(s_null_vec, max(tmp_n_samples_vec, 1), 'linewidth', 2); hold on;
        x_labels = get(gca, 'xticklabel');
        c1 = get(h1, 'color');
        
        [~, mid_ind] = min(abs(s_null_vec - 10^(-3)));
        for j=1:size(tmp_n_samples_vec, 1) % New: Add text
            text(10^(-3), tmp_n_samples_vec(j, mid_ind)*2, [num2str(show_grr_vec(j)')], 'color', c1{j}, 'fontsize', 14);
        end
        
        
        
        xlabel('Selection coefficient s', 'fontsize', 14);
        % title(['Sample size required for XXX% power. \alpha=' ...
        %    num2str(num_samples_struct{4}.p_val_cutoff) '. \pi=' num2str(prevalence) '. ' ...
        %    demographic_models_struct.model_str{i_d}]);
        xlim([10^(-4) 0.1]); % show only strong selection
        if(plot_type==0)
            ylim([100 10^(6.1)]); ylabel('Number of cases', 'fontsize', 14);
        else
            ylabel('Sample Size \times Var. Explained');
        end
        set(gca, 'fontsize', 14);
        add_faint_grid(0.5);
        %    set(gca, 'xtick', 10.^(-3.5:0.5:-1));
        
        %     h_leg = legend(cellstr([repmat('1+\lambda=', length(show_grr_vec), 1) num2str(show_grr_vec')]), 2);
        %     set(h_leg,'Xcolor',[0.8 0.8 0.8],'Ycolor',[0.8 0.8 0.8]);
        
        % No title for final figures
        %    title(['$N_{' num2str(desired_power*100) '\%}$ for different %\lambda%. $p_{\alpha}=' num2str(num_samples_struct{4}.p_val_cutoff) '$. $\pi=' num2str(prevalence) '$. ' ...
        %        demographic_models_struct.model_str{i_d}  '. ' allele_str ' alleles. stat= LRT'],  'interpreter', 'latex'); % (solid) Eric (dashed)'], 'interpreter', 'latex');
        
        
        % New: add another x-axis for f !!!
        ax1=gca; x_ticks1 = get(gca, 'xtick');
        ax2=axes('Position',get(gca,'Position'),...
            'XAxisLocation','top',...
            'YAxisLocation','right',...
            'Color','none',...
            'XColor','k','YColor',[0.8 0.8 0.8]);
        x_labels = neutral_freq_per_gene .* two_class_stat_struct{1}.normalization_factor_x(show_s_null_ind)'; %  x_labels(1) = 0.4;
        hl2=line(x_ticks1, zeros(length(x_ticks1), 1),'Color',[0.8 0.8 0.8],'Parent',ax2); % why do we need this line?
        set(ax2, 'xtick', 0:(0.1/6):0.1);
        x_labels_str = num2str(x_labels', 1);
        x_labels_str = cellstr([x_labels_str(1,:)' x_labels_str']'); % Just repeat first entry (for neutrals)
        set(ax2, 'xticklabel', x_labels_str(end-6:end), 'fontsize', 14); % cellstr(num2str(x_labels(end:-1:1), 2)));
        %   format_ticks(ax2, x_labels_str(end-6:end)); % {'1e-3.5', '1e-3','1e-2.5','1e-2','1e-1.5','10^{-1}'})
        xlabel('f_{null}'); %, 'fontsize', 14);
        
        %  NEED SOME MANUAL TWEAKING TO SHOW f_null BEFORE SAVING!!
        my_saveas(gcf, fullfile(new_figs_dir, 'power/', ...
            [demographic_models_struct.model_str{i_d} '_sample_size_' stat_used protective_str]), ...
            {'epsc', 'pdf', 'jpg'});
        
    end % loop on harmful/protective alleles
end % loop on plot type


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Internal Plot Function 7 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function plot_sample_size_all_models_together_internal(R_all_models, show_s_null, show_s_null_ind, n_null_vec_equilibrium, new_figs_dir)

AssignGeneralConstants;

my_mkdir(fullfile(new_figs_dir, 'all_models', 'individual_stats'));
% Here need two x-axis: s and f !! how to do two axis in log-plot?
for i_plot=1:length(R_all_models) % loop on different values of ..
    save_i = i_plot
    cur_save_file = fullfile(new_figs_dir, 'all_models', 'individual_stats', ...  % PROBLEM WITH FILE NAME !!!
        [strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(strrep(R_all_models{i_plot}{1,1}, ...
        '\', '_'), ' ', '_'), '(', '_'), ')', '_'), '%', '_pct'), '<=', '_less'), '/', '_'), ':', '_'), ...
        '$', ''), '{', ''), '}', ''), '^', '_'), '*', '_'), '_power_table.txt'])
    non_empty_inds = find(~isempty_cell(R_all_models{i_plot}(:,1)));
    savecellfile(num2str_cell(R_all_models{i_plot}(non_empty_inds,:)), cur_save_file,  [], 1);
    
    if(i_plot==18) % Also plot 'pure' sample size
        cur_plot_mat = cell2mat(str2nums_cell(R_all_models{i_plot}(3:end,2:end), 1));
        
        cur_model_legend = R_all_models{i_plot}(2+find(~isempty_cell(R_all_models{i_plot}(3:end,1))),1);
        
        bad_inds = strmatch('varsel', cur_model_legend); good_inds = setdiff(1:length(cur_model_legend), bad_inds); % remove variable selection
        cur_model_legend = cur_model_legend(good_inds);
        cur_plot_mat = cur_plot_mat(good_inds,:);
        
        
        figure;
        for j=1:size(cur_plot_mat, 1)
            switch cur_model_legend{j}
                case {'equil', 'expan1', 'expan2', '2phase'}
                    cur_symbol_vec = '-';
                case {'ice', 'finn1', 'finn2', 'europ'}
                    cur_symbol_vec = '--';
                case {'varsel1', 'varsel2'}
                    cur_symbol_vec = ':';
            end
            loglog(show_s_null, cur_plot_mat(j,1:end)', [color_vec(j) cur_symbol_vec], 'linewidth', 2); hold on; % no need to flip power
        end
        xlabel('s', 'fontsize', 14); ylabel('Sample size', 'fontsize', 14); %title('Sample Size for different models. Prevalence=0.05. GRR=5. Power=50%');
        h_leg = legend(cur_model_legend, 2); % legend('boxoff');
        set(h_leg,'Xcolor',[0.8 0.8 0.8],'Ycolor',[0.8 0.8 0.8]);
        add_faint_grid(0.5);
        x_ticks = get(gca, 'xtick'); x_tick_labels = get(gca, 'xticklabel');
        
        
        % New: add another x-axis with f
        ax1=gca;
        %        set(gca, 'xtick', show_s_null);
        ax2=axes('Position',get(gca,'Position'),...
            'XAxisLocation','top',...
            'YAxisLocation','right',...
            'Color','none',...
            'XColor','k','YColor',[0.8 0.8 0.8]);
        x_labels = n_null_vec_equilibrium(show_s_null_ind([1 2:2:end])); x_labels(1) = 0.4;
        hl2=line(-n_null_vec_equilibrium, zeros(length(n_null_vec_equilibrium), 1),'Color',[0.8 0.8 0.8],'Parent',ax2);
        set(ax2,'xscale','log');
        set(ax2, 'xtick', [-1 -x_labels(2:end-1)' -0.00012]);
        set(ax2, 'xticklabel', cellstr(num2str(x_labels, 2)));
        xlabel('n_{null}');
        
        
        % Add additional x-axis for n_null
        my_saveas(gcf, fullfile(new_figs_dir, 'all_models', 'sample_size_different_models'), {'epsc', 'pdf'});
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  New: compute optimal threshold and power analytically %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Plot inflation factor using two different methods for equilibtium
function plot_inflation_factor_equilibrium_analytic_internal(s_null_vec, N, two_class_stat_struct, ...
    polyphen_false_pos, polyphen_false_neg, ... % set accuracy of functional-allele prediction
    new_figs_dir)

AssignGeneralConstants;

lambda_vec=[0.1 3 5 10]; %  30 100]; % effect size (on disease) of LOF alleles: 1+lambda
theta=0.08; % total frequency of LOF alleles

num_s = length(s_null_vec);
inflation_ctr=1; optimal_threshold_vec = cell(2,1); optimal_power_vec = cell(2,1);
alpha_vec = 0.25; % SET DEFAULT FOR PAPER 0.33; % set fraction at birth
for inflation_factor_type =  {'LRT', 'crude-eric'} % 'LRT';
    optimal_threshold_vec{inflation_ctr} = zeros(num_s, 1);
    optimal_power_vec{inflation_ctr} = zeros(num_s, 1);
    for i_s=1:length(s_null_vec)
        run_s = i_s
        
        switch inflation_factor_type{1}
            case 'LRT'
                for i_lambda=1:length(lambda_vec)
                    lambda = lambda_vec(i_lambda);
                    [optimal_threshold_vec{inflation_ctr}(i_s,i_lambda) optimal_power_vec{inflation_ctr}(i_s,i_lambda)] = fminbnd(@(x) ...
                        -relative_entropy_power_allele_freq_internal(x, s_null_vec(i_s), N, 4*theta, alpha_vec, lambda), ... % take 4*theta for missense
                        0, 0.99999999);
                    
                    P = theta .* (phi_s_integral(0.99999999, -4.*N.*s_null_vec(i_s), 1) - phi_s_integral(0, -4.*N.*s_null_vec(i_s), 1));
                    Q = (1+lambda)*P;
                    
                    
                    %                     ELODs_MISSENSE_exact = optimal_power_vec{inflation_ctr}(i_s,i_lambda)
                    %                     ELODs_LOF_exact = relative_entropy([Q 1-Q], [P 1-P])
                    optimal_power_vec{inflation_ctr}(i_s,i_lambda) = optimal_power_vec{inflation_ctr}(i_s,i_lambda) ./ ...
                        relative_entropy([Q 1-Q], [P 1-P]); % normalize by power of taking all missense
                    
                    rho_vec = frac_null_conditional_on_freq_less_f(-s_null_vec(i_s), N, alpha_vec, optimal_threshold_vec{inflation_ctr}(i_s,i_lambda));
                    f_vec = allele_freq_cumulative(optimal_threshold_vec{inflation_ctr}(i_s,i_lambda), -s_null_vec(i_s), N, 0, 'linear', 1);
                    inflation_factor_by_lambda_vec(i_lambda) = ...
                        approximate_ELODs_ratio_internal(f_vec, rho_vec, lambda_vec(i_lambda));
                    
                    %                     ELODs_LOF_approx = P * ( ((1+lambda_vec(i_lambda)).*log(1+lambda_vec(i_lambda))-lambda_vec(i_lambda)) )
                    %                     ELODs_MISSENSE_approx = P * 4 .* f_vec .* ( (1+rho_vec .* lambda_vec(i_lambda)) .* ...
                    %                         log( 1+rho_vec.*lambda_vec(i_lambda) ) - rho_vec .* lambda_vec(i_lambda) ) ./ rho_vec
                    %                     ELODs_MISSENSE_approx / ELODs_LOF_approx
                end
                
            case 'crude-eric'
                [optimal_threshold_vec{inflation_ctr}(i_s) optimal_power_vec{inflation_ctr}(i_s)] = fminbnd(@(x) ...
                    -frac_null_conditional_on_freq_less_f(-s_null_vec(i_s), N, alpha_vec, x) .* ...
                    (phi_s_integral(x, -4.*N.*s_null_vec(i_s), 1) - phi_s_integral(0, -4.*N.*s_null_vec(i_s), 1)), 0, 0.99999999);
                optimal_power_vec{inflation_ctr}(i_s) = optimal_power_vec{inflation_ctr}(i_s) / ...
                    (phi_s_integral(0.99999999, -4.*N.*s_null_vec(i_s), 1) - phi_s_integral(0, -4.*N.*s_null_vec(i_s), 1)); % normalize
        end % switch inflation type
        
    end % loop on selection
    switch inflation_factor_type{1}
        case 'crude-eric'
            optimal_power_vec{inflation_ctr} = -4 .* optimal_power_vec{inflation_ctr}; % take into account null-missense/LOF ratio
        case 'LRT'
            optimal_power_vec{inflation_ctr} = -optimal_power_vec{inflation_ctr}; % take into account null-missense/LOF ratio
    end
    inflation_ctr=inflation_ctr+1;
end

figure;
semilogx(s_null_vec, two_class_stat_struct.mean_x, 'c', 'linewidth', 2); hold on;
semilogx(s_null_vec, two_class_stat_struct.median_x, 'k', 'linewidth', 2);
semilogx(s_null_vec, optimal_threshold_vec{2}, 'm', 'linewidth', 2);
legend_vec = {'mean-allele-freq', 'median-allele-freq', 'T^* crude'};
optimal_threshold_vec{1}(:,end) = max(optimal_threshold_vec{1}(:,end-1), optimal_threshold_vec{1}(:,end));
for i_lambda=1:length(lambda_vec)
    semilogx(s_null_vec, optimal_threshold_vec{1}(:,i_lambda), [color_vec(i_lambda) '--'], 'linewidth', 2);
    legend_vec{i_lambda+3} = ['T^* KL. \lambda=' num2str(lambda_vec(i_lambda))];
end
xlabel('Selection coefficients, s');
ylabel('Optimal threshold T^*');
legend(legend_vec);


my_saveas(gcf, fullfile(new_figs_dir, 'optimal_threshold_equilibrium_analytic'), {'epsc', 'pdf'});

figure;  semilogx(s_null_vec, optimal_power_vec{2}, 'm', 'linewidth', 2);  hold on;

for i_lambda=1:length(lambda_vec)
    semilogx(s_null_vec, optimal_power_vec{1}(:,i_lambda), color_vec(i_lambda), 'linewidth', 2);
end
legend(legend_vec(3:end));
xlabel('Selection coefficients, s');
ylabel('Optimal SNR (compared to LOF)*');
ylim([0.5, 4]);
xlim([10^(-4) 10^(-1)]);
for i_lambda=1:length(lambda_vec) % plot crude bounds
    semilogx(s_null_vec, repmat(inflation_factor_by_lambda_vec(i_lambda), length(s_null_vec), 1), [color_vec(i_lambda) '--'], 'linewidth', 2);
end

my_saveas(gcf, fullfile(new_figs_dir, 'optimal_power_equilibrium_analytic'), {'epsc', 'pdf'});


% New: plot analytic expression of power inflation as function of lambda
lambda_vec = 10000:10000:100000000;
f_vec = 0.8; rho_vec = 0.4; % take sample values
inflation_factor_by_lambda_vec = 4 .* f_vec .* ( (1+rho_vec .* lambda_vec) .* log( 1+rho_vec.*lambda_vec ) - rho_vec .* lambda_vec ) ./ ...
    ( rho_vec .* ((1+lambda_vec).*log(1+lambda_vec)-lambda_vec) );

figure; plot(lambda_vec, inflation_factor_by_lambda_vec); hold on;
plot(lambda_vec, repmat(4 .* f_vec .* rho_vec, length(lambda_vec), 1), 'k--', 'linewidth', 2); % lower-bound: small lambda
plot(lambda_vec, repmat(4 .* f_vec, length(lambda_vec), 1), 'k--', 'linewidth', 2); % upper-bound: large lambda
xlabel('\lambda'); ylabel('inflation-factor(\lambda)');


% New: surfact plots (should move this to another figure)
all_rho_vec = logspace(-6,0, 500); all_psi_vec = logspace(-6,0,600); % 500 to make it smoother
num_rho = length(all_rho_vec); num_psi = length(all_psi_vec);


alpha_vec=0.25; % SET DEFAULT FOR PAPER [0.33]; %  0.5 0.7];
lambda=3; % 0.1;

ELODs_ratio_mat = approximate_ELODs_ratio_internal(repmat(all_psi_vec, num_rho, 1), repmat(all_rho_vec', 1, num_psi), lambda); % compute one time in matrix form
% ELODs_ratio_mat = zeros(num_rho, num_psi);
% for i=1:length(all_rho_vec)
%     if(mod(i,10)==0)
%         i_is = i
%     end
%     for j=1:length(all_psi_vec)
%         % Compute power matrix
%         ELODs_ratio_mat(i,j) = approximate_ELODs_ratio_internal(all_psi_vec(j), all_rho_vec(i), lambda);
%
%     end
% end


for plot_polyphen=0:1
    figure; % contours
    
    
    s=0.01; N=10000;
    f_vec = logspace(-6,0,5000); % possible allele frequencies
    hidden_rho_vec = zeros(length(f_vec), length(alpha_vec));
    hidden_psi_vec = zeros(length(f_vec), length(alpha_vec));
    for i=1:length(f_vec)
        for j=1:length(alpha_vec)
            if(mod(i,50)==0)
                run_i=i
            end
            hidden_psi_vec(i,j) = allele_freq_cumulative(f_vec(i), -s, N, 0, [], 1);
            hidden_rho_vec(i,j) = frac_null_conditional_on_freq_less_f(-s, N, alpha_vec(j), f_vec(i));
        end % loop on alpha
    end
    
    
    
    [optimal_threshold optimal_power] = fminbnd(@(x) ...
        -relative_entropy_power_allele_freq_internal(x, s, N, 4*theta, alpha_vec, lambda), ... % take 4*theta for missense
        0, 0.99999999);
    optimal_psi = allele_freq_cumulative(optimal_threshold, -s, N, 0, [], 1);
    optimal_rho = frac_null_conditional_on_freq_less_f(-s, N, alpha_vec, optimal_threshold);
    optimal_ELODs = approximate_ELODs_ratio_internal(optimal_psi, optimal_rho, lambda);
    
    [optimal_threshold_polyphen optimal_power_polyphen] = fminbnd(@(x) ...
        -relative_entropy_power_allele_freq_internal(x, s, N, 4*theta, alpha_vec, lambda, [polyphen_false_pos polyphen_false_neg]), ... % take 4*theta for missense
        0, 0.99999999);
    optimal_psi_polyphen = allele_freq_cumulative(optimal_threshold_polyphen, -s, N, 0, [], 1);
    optimal_rho_polyphen = frac_null_conditional_on_freq_less_f(-s, N, alpha_vec, optimal_threshold_polyphen);
    optimal_ELODs_polyphen = approximate_ELODs_ratio_internal((1-polyphen_false_neg) .* optimal_psi_polyphen, ...
        (1-polyphen_false_neg) .* optimal_rho_polyphen ./ ( (1-polyphen_false_neg) .* optimal_rho_polyphen + polyphen_false_pos .* (1-optimal_rho_polyphen) ), lambda);
    
    max_y_val = ((1-polyphen_false_neg) ./ polyphen_false_pos) .* hidden_rho_vec(1) ./ (1-hidden_rho_vec(1));
    good_rho_inds = find(hidden_rho_vec ./ (1-hidden_rho_vec) <= ...
        1.1 .* max_y_val); % Allow at most 2/3 nulls
    for j=1:length(alpha_vec)
        plot(hidden_psi_vec(good_rho_inds), ...
            hidden_rho_vec(good_rho_inds) ./ (1-hidden_rho_vec(good_rho_inds)), 'b--', 'linewidth', 2); hold on; % Plot with new axis
        hold on;
        if(plot_polyphen)
            plot( (1-polyphen_false_neg) .* hidden_psi_vec(good_rho_inds), ...
                ((1-polyphen_false_neg) ./ polyphen_false_pos) .* hidden_rho_vec(good_rho_inds) ./ (1-hidden_rho_vec(good_rho_inds)), 'r--', 'linewidth', 2); % New! add polyphen. Also plot new axis
        end
    end
    
    plot(optimal_psi, optimal_rho ./ (1-optimal_rho), '*b', 'markersize', 10);
    plot(optimal_psi, optimal_rho ./ (1-optimal_rho), 'ob', 'markersize', 10);
    if(plot_polyphen)
        plot((1-polyphen_false_neg) .* optimal_psi_polyphen, ...
            ((1-polyphen_false_neg) ./ polyphen_false_pos) .* optimal_rho_polyphen ./ (1-optimal_rho_polyphen), '*r', 'markersize', 10);
        plot((1-polyphen_false_neg) .* optimal_psi_polyphen, ...
            ((1-polyphen_false_neg) ./ polyphen_false_pos) .* optimal_rho_polyphen ./ (1-optimal_rho_polyphen),  'or', 'markersize', 10);
    end
    
    good_rho_inds = find(all_rho_vec ./ (1-all_rho_vec) <= ...
        1.1 .* max_y_val); % Allow at most 2/3 nulls
    if(good_rho_inds(end) < length(all_rho_vec))
        good_rho_inds = [good_rho_inds good_rho_inds(end)+1]; % add one more index to contour plot
    end
    colormap('gray'); [c, h] = contour(all_psi_vec, all_rho_vec(good_rho_inds) ./ (1-all_rho_vec(good_rho_inds)), ...
        ELODs_ratio_mat(good_rho_inds,:), 'linewidth', 2); clabel(c, h); colorbar;
    
    
    ylim([0 1.1*max_y_val]);
    
    if(plot_polyphen)
        legend({['all alleles: I(T^*)=' num2str(optimal_ELODs,2)] , ['func. prediction(20%, 20%): I(T^*)='  num2str(optimal_ELODs_polyphen,3)]}, 3); legend('boxoff');
    end
    xlabel('Frac. captured. \Psi_s(f)'); ylabel('Null vs. Neutral proportion: \rho_s(f) / (1-\rho_s(f))');
    %title(['E[LODs-missense] / E[LODs-LOF] for equilibrium. \lambda=0.1. s=0.01. \alpha=' num2str(alpha_vec)]);
    
    if(plot_polyphen)
        my_saveas(gcf, fullfile(new_figs_dir, 'optimal_power_equilibrium_contour_with_polyphen_analytic'), {'epsc', 'pdf', 'jpg'});
    else
        my_saveas(gcf, fullfile(new_figs_dir, 'optimal_power_equilibrium_contour_analytic'), {'epsc', 'pdf', 'jpg'});
    end
end % loop on plot-polyphen

% Compute relative entropy between frequency of counted alleles in the population and in cases
function ret = relative_entropy_power_allele_freq_internal(x, s, N, theta, alpha_vec, lambda, polyphen_errors)

if(exist('polyphen_errors', 'var') && ~isempty(polyphen_errors))
    polyphen_false_pos=polyphen_errors(1);
    polyphen_false_neg=polyphen_errors(2);
end

S=4*N*s;
rho = frac_null_conditional_on_freq_less_f(-s, N, alpha_vec, x);
psi_unnormalized = (phi_s_integral(x, -S, 1) - phi_s_integral(0, -S, 1));
if(~exist('polyphen_errors', 'var') || isempty(polyphen_errors))
    
else % correct according to polyphen
    rho = rho .* (1-polyphen_false_neg) ./ (rho .* (1-polyphen_false_neg) + (1-rho) .* polyphen_false_pos);
    psi_unnormalized = psi_unnormalized .* (1-polyphen_false_neg);
end

P = theta .* psi_unnormalized ./ rho;  ... %     (phi_s_integral(0.9999999, -S, 1) - phi_s_integral(0, -S, 1))
    Q = (1 + lambda .* rho) .* P;
ret = relative_entropy([vec2column(Q) 1-vec2column(Q)]', [vec2column(P) 1-vec2column(P)]');


% Compute approximate ratio of ELODs between missense and LOFs. This is
% Taylor series expansion of the relative entropy for small values of s.
%
% Input:
% f - fraction of nulls captured
% rho - fraction of alleles which are null
% lambda - effect size
%
function ret = approximate_ELODs_ratio_internal(f, rho, lambda)

ret = 4 .* f .* ( (1+rho .* lambda) .* ...
    log( 1+rho.*lambda) - rho .* lambda) ./ ...
    ( rho .* ((1+lambda).*log(1+lambda)-lambda) );










% Compute optimal ELODs ratio for different models
function output_demographic_models_struct = compute_inflation_factor_internal(demographic_models_struct, i_d, ...
    polyphen_false_pos, polyphen_false_neg, ...% set accuracy of functional-allele prediction
    frac_null_by_freq_cumulative, c_cumulative, alpha_vec, beta_hypomorph, delta_hypomorph)

[mu gene_length mu_per_gene] = set_genomic_mutation_rates(alpha_vec); % no beta for now !!!

switch demographic_models_struct.model_str{i_d}
    case 'equilibrium'
        demographic_models_struct.data{i_d}.x_vec = f_rare_vec;
        demographic_models_struct.data{i_d}.rho_vec = frac_null_by_freq_cumulative{1}(show_s_null_ind,:); % fraction of null alleles conditional on allele freq. below f
        demographic_models_struct.data{i_d}.f_null_vec = c_cumulative{1}(show_s_null_ind,:); % fraction of null alleles which are below freq. f
    otherwise % demographic simulations
        demographic_models_struct.data{i_d} = load(demographic_models_struct.file_names{i_d}); % this gives both s-vec and cumulatives
        num_s = size ( demographic_models_struct.data{i_d}.p_vec, 1 );
        demographic_models_struct.data{i_d}.rho_vec = alpha_vec .* demographic_models_struct.data{i_d}.p_vec(end:-1:1,:) ./ ...
            ( alpha_vec .* demographic_models_struct.data{i_d}.p_vec(end:-1:1,:) + ...
            (1-alpha_vec) .* repmat(demographic_models_struct.data{i_d}.p_vec(end,:), num_s, 1) ); % last value is neutral
        demographic_models_struct.data{i_d}.f_null_vec = bsxfun(@rdivide, demographic_models_struct.data{i_d}.p_vec(end:-1:1,:), ...
            demographic_models_struct.data{i_d}.p_vec(end:-1:1,end));  % make sure this is normalized. This should go UP!
end

two_class_data_dir = '../../common_disease_model/data/demographic_simulations/EuropeFixed/fine_s_var_f_disruptive_1_7_10_6/out_var/mat';
FINE = load(fullfile(two_class_data_dir, 'all', 'all_models_statistics.mat')); % load all data
FINE.num_alleles_per_chrom = FINE.mean_vec ./ (0.17); % try - how to normalize? nulls vs. disruptives


inflation_factor_type =  'LRT'; % 'crude-eric'; % 'LRT';
switch inflation_factor_type
    case 'crude-eric'
        demographic_models_struct.data{i_d}.sample_size_inflation_factor =  ...
            1 ./ (demographic_models_struct.data{i_d}.rho_vec .* demographic_models_struct.data{i_d}.f_null_vec); % Says how much is sample size increased
    case 'LRT'
        lambda_vec = [0.1 0.3 0.5 0.75 1 1.5 2 3 5 8 10 15 20]; % get a dense array of lambda values
        %        lambda_vec = [0.1 0.5 1 3 10]; % possible effect sizes
        p_vec = 0.02 .* demographic_models_struct.data{i_d}.f_null_vec .* ...
            repmat( vec2column(demographic_models_struct.data{i_d}.num_alleles_per_chrom(end:-1:1)), ...
            1, length(demographic_models_struct.data{i_d}.f_null_vec)); % 0.2 is for LOF. Why 0.02?  JUST TO MAKE EVERYTHING SMALL
        demographic_models_struct.data{i_d}.lambda_vec = lambda_vec;
        for i_lambda=1:length(lambda_vec)
            q_vec = p_vec .* (lambda_vec(i_lambda)+1);
            LRT_nulls = q_vec(:,end) .* log(q_vec(:,end) ./ p_vec(:,end)) + (1-q_vec(:,end)) .* log( (1-q_vec(:,end)) ./ (1-p_vec(:,end)) ); % LRT for nulls. Take all of them
            LRT_nulls = repmat(LRT_nulls, 1, length(demographic_models_struct.data{i_d}.x_vec));
            
            p_vec_with_missense = (mu_per_gene.MISSENSE_NULL / mu_per_gene.DISRUPTIVE) .* ...  % NEW! replaced 4 by 7.3*alpha !!!
                p_vec ./ demographic_models_struct.data{i_d}.rho_vec; % assume 4 times as many missense than nulls
            q_vec_with_missense = p_vec_with_missense .* ...
                (1 + lambda_vec(i_lambda) .* demographic_models_struct.data{i_d}.rho_vec);
            LRT_with_missense =  q_vec_with_missense .* log(q_vec_with_missense ./ p_vec_with_missense) + ...
                (1-q_vec_with_missense) .* log( (1-q_vec_with_missense) ./ (1-p_vec_with_missense) ); % LRT for missense% Here frequency is 5 times larger. But effect size is dilluted
            demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda} = ...
                LRT_with_missense ./ LRT_nulls;  % Says how much is sample size increased
            [~, demographic_models_struct.data{i_d}.optimal_threshold_inds(i_lambda,:)] = ...
                max(demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda},[],2);
            demographic_models_struct.data{i_d}.optimal_threshold(i_lambda,:) = ...
                demographic_models_struct.data{i_d}.x_vec(demographic_models_struct.data{i_d}.optimal_threshold_inds(i_lambda,:)); % compute optimal threshold for test
            
            % NEW: Power with hypomorph alleles
            europ_inds = strmatch('europ', FINE.pop_vec);
            fine_s_vec = FINE.s_vec(europ_inds);
            s_vec = demographic_models_struct.data{i_d}.s_vec;
            dilluted_s_vec = s_vec .* delta_hypomorph; % Compute dilluted selection coefficient
            for j=1:length(dilluted_s_vec) % find closest index
                [~, dilluted_index_vec(j)] = min(abs(fine_s_vec - dilluted_s_vec(j)));  % FILL!!!
            end
            
            
            p_vec_for_hypomorph = 0.02 .* cell2mat(FINE.f_null_vec(europ_inds(dilluted_index_vec))') .* ...
                repmat(FINE.num_alleles_per_chrom(europ_inds(dilluted_index_vec))', 1, length(demographic_models_struct.data{i_d}.f_null_vec)); % 0.2 is for LOF. Why 0.02?
            if(size(p_vec_for_hypomorph, 1)== 10)
                p_vec_for_hypomorph = p_vec_for_hypomorph(10:-1:1,:); % flip selection order
            end
            p_vec_with_missense_hypomorph = (beta_hypomorph .* mu_per_gene.MISSENSE / mu_per_gene.DISRUPTIVE) .* ...
                p_vec_for_hypomorph ./ (2*demographic_models_struct.data{i_d}.rho_vec); % Problem !! here we also need to adjust selection coefficient !!!
            %             p_vec_with_missense_hypomorph = (beta_hypomorph .* mu_per_gene.MISSENSE / mu_per_gene.DISRUPTIVE) .* ...
            %                 p_vec(dillted_index_vec,:) ./ demographic_models_struct.data{i_d}.rho_vec; % Problem !! here we also need to adjust selection coefficient !!!
            q_vec_with_missense_hypomorph = p_vec_with_missense_hypomorph .* ... % lower effect size !!!
                (1 + delta_hypomorph .* lambda_vec(i_lambda) .* 2.*demographic_models_struct.data{i_d}.rho_vec); % twice  rho! (beta=2*alpha)
            LRT_with_missense_hypomorph =  q_vec_with_missense_hypomorph .* log(q_vec_with_missense_hypomorph ./ p_vec_with_missense_hypomorph) + ...
                (1-q_vec_with_missense_hypomorph) .* log( (1-q_vec_with_missense_hypomorph) ./ (1-p_vec_with_missense_hypomorph) ); % LRT for missense hypomorph % Here frequency is 5 times larger. But effect size is dilluted
            demographic_models_struct.data{i_d}.sample_size_inflation_factor_hypomorph{i_lambda} = ...
                LRT_with_missense_hypomorph ./ LRT_nulls;  % Says how much is sample size increased
            [~, demographic_models_struct.data{i_d}.optimal_threshold_inds_hypomorph(i_lambda,:)] = ...
                max(demographic_models_struct.data{i_d}.sample_size_inflation_factor_hypomorph{i_lambda},[],2);
            demographic_models_struct.data{i_d}.optimal_threshold_hypomorph(i_lambda,:) = ...
                demographic_models_struct.data{i_d}.x_vec(demographic_models_struct.data{i_d}.optimal_threshold_inds_hypomorph(i_lambda,:)); % compute optimal threshold for test
            
            demographic_models_struct.data{i_d}.sample_size_inflation_factor_missense_null_plus_hypomorph{i_lambda} = ...
                (LRT_with_missense_hypomorph+LRT_with_missense) ./ LRT_nulls;  % Says how much is sample size increased
            [~, demographic_models_struct.data{i_d}.optimal_threshold_inds_missense_null_plus_hypomorph(i_lambda,:)] = ...
                max(demographic_models_struct.data{i_d}.sample_size_inflation_factor_missense_null_plus_hypomorph{i_lambda},[],2);
            demographic_models_struct.data{i_d}.optimal_threshold_missense_null_plus_hypomorph(i_lambda,:) = ...
                demographic_models_struct.data{i_d}.x_vec(demographic_models_struct.data{i_d}.optimal_threshold_inds_missense_null_plus_hypomorph(i_lambda,:)); % compute optimal threshold for test
            
            
            
            
            % NEW: Power with polyphen
            p_vec_with_missense_polyphen = p_vec_with_missense .* ...
                (demographic_models_struct.data{i_d}.rho_vec .* (1-polyphen_false_neg) + (1-demographic_models_struct.data{i_d}.rho_vec) .* polyphen_false_pos); % assume 4 times as many missense than nulls
            q_vec_with_missense_polyphen = p_vec_with_missense_polyphen .* (1 + ...
                lambda_vec(i_lambda) .* demographic_models_struct.data{i_d}.rho_vec .* (1-polyphen_false_neg) ./ (demographic_models_struct.data{i_d}.rho_vec .* (1-polyphen_false_neg) + (1-demographic_models_struct.data{i_d}.rho_vec) .* polyphen_false_pos)   );
            LRT_with_missense_polyphen =  q_vec_with_missense_polyphen .* log(q_vec_with_missense_polyphen ./ p_vec_with_missense_polyphen) + ...
                (1-q_vec_with_missense_polyphen) .* log( (1-q_vec_with_missense_polyphen) ./ (1-p_vec_with_missense_polyphen) ); % LRT for missense with polyphen% Here frequency is 5 times larger. But effect size is dilluted
            demographic_models_struct.data{i_d}.sample_size_inflation_factor_polyphen{i_lambda} = LRT_with_missense_polyphen ./ LRT_nulls;  % Says how much is sample size increased
            [~, demographic_models_struct.data{i_d}.optimal_threshold_inds_polyphen(i_lambda,:)] = ...
                max(demographic_models_struct.data{i_d}.sample_size_inflation_factor_polyphen{i_lambda},[],2);
            demographic_models_struct.data{i_d}.optimal_threshold_polyphen(i_lambda,:) = ...
                demographic_models_struct.data{i_d}.x_vec(demographic_models_struct.data{i_d}.optimal_threshold_inds_polyphen(i_lambda,:)); % compute optimal threshold for test
            
            for i_s=1:10 % Compute for each selection coefficient
                demographic_models_struct.data{i_d}.optimal_rho(i_lambda,i_s) = ... % compute optimal threshold for test
                    demographic_models_struct.data{i_d}.rho_vec(i_s,demographic_models_struct.data{i_d}.optimal_threshold_inds(i_lambda,i_s));
                demographic_models_struct.data{i_d}.optimal_f_vec(i_lambda,i_s) = ... % compute optimal threshold for test
                    demographic_models_struct.data{i_d}.f_null_vec(i_s,demographic_models_struct.data{i_d}.optimal_threshold_inds(i_lambda,i_s));
                demographic_models_struct.data{i_d}.optimal_inflation_factor(i_lambda,i_s) = ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor{i_lambda}(i_s, demographic_models_struct.data{i_d}.optimal_threshold_inds(i_lambda,i_s));
                
                
                % NEW! Also compute optimal inflation factor when using polyphen
                demographic_models_struct.data{i_d}.optimal_inflation_factor_polyphen(i_lambda,i_s) = ...
                    demographic_models_struct.data{i_d}.sample_size_inflation_factor_polyphen{i_lambda}(i_s, demographic_models_struct.data{i_d}.optimal_threshold_inds_polyphen(i_lambda,i_s));
                
                demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda,i_s) = ... % compute optimal threshold for test
                    demographic_models_struct.data{i_d}.rho_vec(i_s,demographic_models_struct.data{i_d}.optimal_threshold_inds_polyphen(i_lambda,i_s));
                demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda,i_s) = ...
                    (1-polyphen_false_neg) .* demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda,i_s) ./ ...
                    ( (1-polyphen_false_neg) .* demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda,i_s) + ...
                    polyphen_false_pos .* (1-demographic_models_struct.data{i_d}.optimal_rho_polyphen(i_lambda,i_s)) );
                demographic_models_struct.data{i_d}.optimal_f_vec_polyphen(i_lambda, i_s) = ...% NEW! add polyphen
                    (1-polyphen_false_neg) .* demographic_models_struct.data{i_d}.f_null_vec(i_s,demographic_models_struct.data{i_d}.optimal_threshold_inds_polyphen(i_lambda,i_s));
                
                
                [~, demographic_models_struct.data{i_d}.median_DAF_ind(i_s)] = min(abs(demographic_models_struct.data{i_d}.f_null_vec(i_s,:) - 0.5));
                demographic_models_struct.data{i_d}.median_DAF(i_s) = ...
                    demographic_models_struct.data{i_d}.x_vec(demographic_models_struct.data{i_d}.median_DAF_ind(i_s));
            end % loop on selection coefficient s
        end % loop on effect size lambda
end % switch inflation factor

output_demographic_models_struct = demographic_models_struct;



% Plot de-novo power. Here results do not depend on selection
function plot_two_class_de_novo_power(mu_vec, alpha, beta, lambda_vec, new_figs_dir)

% Get NCP
NCP = alpha_and_beta_to_non_centrality_parameter(alpha, beta);

n_samples_vec = zeros(length(lambda_vec), length(mu_vec));

for i=1:length(lambda_vec)
    n_samples_vec(i,:) = NCP ./ (4.*mu_vec .* ((1+lambda_vec(i)).*log(1+lambda_vec(i))-lambda_vec(i)));
end

figure; h1 = loglog(mu_vec, n_samples_vec, 'linewidth', 2); hold on;
c1 = get(h1, 'color');

[~, mid_ind] = min(abs(mu_vec - 10^(-6)));


for i=1:length(lambda_vec) % New: Add text
    text(10^(-6), n_samples_vec(i, mid_ind)*1.3, [num2str(lambda_vec(i)'+1)], 'color', c1{i}, 'fontsize', 14);
end
text(10^(-6.23), n_samples_vec(1, mid_ind)*2.2, '1+\lambda=', 'color', 'k', 'fontsize', 14);
xlabel('\mu_D'); ylabel('Number of cases');

add_faint_grid(0.5);
my_saveas(gcf, fullfile(new_figs_dir, 'de_novo_power'), {'epsc', 'pdf', 'jpg'});



% Plot power of disruptive. Here results DO depend on selection
function plot_diruptive_power(mu, s_vec, lambda_vec, alpha, beta, new_figs_dir)


mu=1.7*10^(-6);

% Get NCP
NCP = alpha_and_beta_to_non_centrality_parameter(alpha, beta);

n_samples_vec = zeros(length(lambda_vec), length(s_vec));

N=10000;
%f_vec = 4.*N.*mu*exp(allele_freq_cumulative(1-1/(2*N), -s_vec, N, 0, 'log', 1));

f_vec = 4.*N.*mu* absorption_time_by_selection(s_vec, 1, N, 0/(2*N), 1-1/(2*N), 1);


for protective_flag = 0:1
    [~, mid_ind] = min(abs(s_vec - 10^(-3)));
    
    if(protective_flag)
        protective_str = '_protective';
        use_lambda_vec = [1 ./ (1+lambda_vec) - 1 -0.9999999];
        lambda_str = '(1+\lambda)^{-1}=';
        lambda_offset_x = 10^(-3.38);
    else
        protective_str = '';
        use_lambda_vec = lambda_vec;
        lambda_str = '1+\lambda=';
        lambda_offset_x = 10^(-3.23);
    end
    for i=1:length(use_lambda_vec)
        n_samples_vec(i,:) = NCP ./ (4.*f_vec .* ((1+use_lambda_vec(i)).*log(1+use_lambda_vec(i))-use_lambda_vec(i)));
    end
    lambda_offset_vec = n_samples_vec(:, mid_ind)*2;
    if(protective_flag) % need to correct height
        lambda_offset_y = n_samples_vec(1, mid_ind)*2.0;
        lambda_offset_vec(1) = lambda_offset_vec(1)*0.9;
        lambda_offset_vec(2) = lambda_offset_vec(2)*0.8;
        lambda_offset_vec(3) = lambda_offset_vec(3)*0.8;
        lambda_offset_vec(4) = lambda_offset_vec(4)*0.7;
        lambda_offset_vec(5) = lambda_offset_vec(5)*0.7;
        lambda_offset_vec(6) = lambda_offset_vec(6)*0.45;
        lambda_offset_vec(7) = lambda_offset_vec(7)*0.4; % NEW: add infinity
    else
        lambda_offset_y = n_samples_vec(1, mid_ind)*2.2;
    end
    
    figure; h1 = loglog(s_vec, n_samples_vec, 'linewidth', 2); hold on;
    c1 = get(h1, 'color');
    
    
    
    xlim([10^(-4) 10^(-1)]); % set only relevant selection
    ylim([10^2, 10^6]);
    
    for i=1:length(lambda_vec) % New: Add text
        text(10^(-3), lambda_offset_vec(i), [num2str(lambda_vec(i)'+1)], 'color', c1{i}, 'fontsize', 14);        
    end
    if(protective_flag)
        text(10^(-3), lambda_offset_vec(end), '$\infty$', 'interpreter', 'latex', 'color', ...
            c1{length(lambda_offset_vec)}, 'fontsize', 14);
    end
    text(lambda_offset_x, lambda_offset_y, lambda_str, 'color', 'k', 'fontsize', 14);
    %xlabel('Selection coefficient, s');
    ylabel('Number of cases', 'fontsize', 14);
    set(gca, 'fontsize', 14)
    
    text(1.6*10^(-1), 0.8*10^2, 's', 'fontsize', 14);
    text(1.6*10^(-1), 0.4*10^2, 'f_D', 'fontsize', 14);
    text(0.67*10^(-4), 0.4*10^2, [num2str(f_vec(200) / 10^floor(log10(f_vec(200))), 2) 'x10^{' num2str(floor(log10(f_vec(200)))) '}'], 'fontsize', 14); % Why are these fixed???? WTF??
    text(0.67*10^(-3), 0.4*10^2, [num2str(f_vec(300) / 10^floor(log10(f_vec(300))), 2) 'x10^{' num2str(floor(log10(f_vec(300)))) '}'], 'fontsize', 14);
    text(0.67*10^(-2), 0.4*10^2, [num2str(f_vec(400) / 10^floor(log10(f_vec(400))), 2) 'x10^{' num2str(floor(log10(f_vec(400)))) '}'], 'fontsize', 14);
    text(0.63*10^(-1), 0.4*10^2, ...
        [num2str(f_vec(end) / 10^floor(log10(f_vec(end))), 2) 'x10^{' num2str(floor(log10(f_vec(end)))) '}'], ...
        'fontsize', 14);
    
    add_faint_grid(0.5);
    my_saveas(gcf, fullfile(new_figs_dir, ['disruptive_power' protective_str]), {'epsc', 'pdf', 'jpg'});
    
end % loop on protective flag

